var getTitleShowHideItem = function(chartId, chart) {
    var item = {
        name: 'Title',
        visible: false
    };
    var itemId = chartId + "title";
    if (!chart.title) {
        if (!localStorage.getItem(itemId))
            return item;
        item.visible = true;
        item.callback = function() {
            chart.setTitle({
                text: localStorage.getItem(itemId)
            })
        };
    } else if (chart.title.styles.display == "none") {
        return item;
    } else {
        item.icon = "fa-check";
        item.visible = true;
        item.callback = function() {
            chart.setTitle({
                text: null
            })
        };
        localStorage.setItem(itemId, chart.title.textStr);
    }
    return item;
}
var getSubTitleShowHideItem = function(chartId, chart) {
    var item = {
        name: 'Sub Title',
        visible: false
    };
    var itemId = chartId + "subtitle";
    if (!chart.subtitle) {
        if (!localStorage.getItem(itemId))
            return item;
        item.visible = true;
        item.callback = function() {
            chart.setTitle(null, {
                text: localStorage.getItem(itemId)
            })
        };
    } else if (chart.subtitle.styles.display == "none") {
        return item;
    } else {
        item.icon = "fa-check";
        item.visible = true;
        item.callback = function() {
            chart.setTitle(null, {
                text: null
            })
        };
        localStorage.setItem(itemId, chart.subtitle.textStr);
    }
    return item;
}
var getLegendShowHideItem = function(chart) {
    var item = {
        name: 'Legend'
    };
    if (!chart.legend) {
        item.visible = false;
    } else if (chart.legend.display) {
        item.icon = "fa-check";
        item.visible = true;
        item.callback = function() {
            chart.legend.update({
                enabled: false
            })
        };
    } else {
        item.visible = true;
        item.callback = function() {
            chart.legend.update({
                enabled: true
            })
        };
    }
    return item;
}
var getToolTipShowHideItem = function(chart) {
    var item = {
        name: 'Tooltip'
    };
    if (!chart.tooltip) {
        item.visible = false;
    } else if (chart.tooltip.options.style.display == "none") {
        item.visible = true;
        item.callback = function() {
            chart.update({
                tooltip: {
                    style: {
                        display: 'block'
                    }
                }
            })
        };
    } else {
        item.icon = "fa-check";
        item.visible = true;
        item.callback = function() {
            chart.update({
                tooltip: {
                    style: {
                        display: 'none'
                    }
                }
            })
        };
    }
    return item;
}
var getDataLabelShowHideItem = function(chart) {
    var item = {
        name: 'Data Labels'
    };
    if (!chart.series) {
        item.visible = false;
    } else if (chart.series.length == 0) {
        item.visible = false;
    } else {
        if (chart.series[0].options.dataLabels.enabled) {
            item.icon = "fa-check";
            item.callback = function() {
                for (series in chart.series) {
                    chart.series[series].update({
                        dataLabels: {
                            enabled: false
                        }
                    });
                }
            };
        } else {
            item.callback = function() {
                for (series in chart.series) {
                    chart.series[series].update({
                        dataLabels: {
                            enabled: true
                        }
                    });
                }
            };
        }
        item.visible = true;
    }
    return item;
}
var getXAxisTitleShowHideItem = function(chartId, chart) {
    var item = {
        name: 'XAxis Title',
        visible: false
    };
    var itemId = chartId + "xaxistitle";
    if (!chart.xAxis) {
        return item;
    } else if (chart.xAxis.length === 0) {
        return item;
    } else if (!chart.xAxis[0].axisTitle) {
        if (!localStorage.getItem(itemId + "0"))
            return item;
        item.visible = true;
        item.callback = function() {
            for (axis in chart.xAxis) {
                chart.xAxis[axis].update({
                    title: {
                        text: localStorage.getItem(itemId + axis)
                    }
                });
            }
        };
    } else if (chart.xAxis[0].axisTitle.styles.display == "none") {
        return item;
    } else {
        item.icon = "fa-check";
        item.visible = true;
        for (axis in chart.xAxis) {
            localStorage.setItem(itemId + axis, chart.xAxis[axis].axisTitle.textStr)
        }
        item.callback = function() {
            for (axis in chart.xAxis) {
                chart.xAxis[axis].update({
                    title: {
                        text: null
                    }
                });
            }
        }
    }
    return item;
}
var getYAxisTitleShowHideItem = function(chartId, chart) {
    var item = {
        name: 'YAxis Title',
        visible: false
    };
    var itemId = chartId + "yaxistitle";
    if (!chart.yAxis) {
        return item;
    } else if (chart.yAxis.length === 0) {
        return item;
    } else if (!chart.yAxis[0].axisTitle) {
        if (!localStorage.getItem(itemId + "0"))
            return item;
        item.visible = true;
        item.callback = function() {
            for (axis in chart.yAxis) {
                chart.yAxis[axis].update({
                    title: {
                        text: localStorage.getItem(itemId + axis)
                    }
                });
            }
        };
    } else if (chart.yAxis[0].axisTitle.styles.display == "none") {
        return item;
    } else {
        item.icon = "fa-check";
        item.visible = true;
        for (axis in chart.yAxis) {
            localStorage.setItem(itemId + axis, chart.yAxis[axis].axisTitle.textStr)
        }
        item.callback = function() {
            for (axis in chart.yAxis) {
                chart.yAxis[axis].update({
                    title: {
                        text: null
                    }
                });
            }
        }
    }
    return item;
}
var getTitleContextAwareItem=function(){
    
}
