var IconExplorer = (function () {
	var instance;
	function IconExplorerSingleton(component, customFontProp,iconExplorerType) {
		this.iconExplorerType = iconExplorerType;
		if (instance) {
			instance.iconExplorerType = iconExplorerType;
			return instance;
		}
		instance = this;
		this.component = component;
		this.callBackStore = {};
		this.customFontProp = customFontProp;
		var _iconPool = new iconExplorerIconPool();
		this.getIconList = _iconPool.getIconList.bind(_iconPool);
		this.getIconInfo = _iconPool.getIconInfo.bind(_iconPool);
		this.addToIconPool = _iconPool.addToIconPool.bind(_iconPool);
		this.validateIconUrl = _iconPool.validateIconUrl.bind(_iconPool);
		//initializing UI and binding events
		this.insertSAPFontFaceStyle();
		this.insertIconExplorerDom(); 
		this.bindEvents();
	}

	IconExplorerSingleton.getInstance = function () {
		return instance || new IconExplorerSingleton();
	}

	IconExplorerSingleton.prototype.bindElement = function (ele, cb) {
		var self = this,
			eleId = ele.attr('id') || ele.attr('class');
		if (eleId && cb && typeof cb === "function") {
			self.callBackStore[eleId] = cb;
		}

		ele.off().on('click', { self: self }, function (event) {
			var self = event.data.self;
			self.boundElement = $(this);
			$("#icon_explorer_input").val('');
			$("#icon_explorer_icontype").val('fa');
			self.renderIconExplorer(self.boundElement.prev().val());
			//$("#icon_explorer_panel").show();
			//$("#icon_explorer_panel_content").show();
			$("div.icon-explorer-modal-aps").show().addClass("in");
			$("body").addClass("modal-open");
			if($("div.modal-backdrop").length === 0){
				$("body").append("<div class='modal-backdrop fade in'></div>");
			}
		});
	};

	IconExplorerSingleton.prototype.insertSAPFontFaceStyle = function () {
		var g = '<style id="ix_sapicons_fontface" type="text/css">@font-face {font-family: "SAP-icons";src: url("/aad/resources/sap/ui/core/themes/base/fonts/SAP-icons.eot");src: url("/aad/resources/sap/ui/core/themes/base/fonts/SAP-icons.eot?#iefix") format("embedded-opentype"),url("/aad/resources/sap/ui/core/themes/base/fonts/SAP-icons.ttf") format("truetype");font-weight: normal;font-style: normal;}</style>';
		if (!$("#ix_sapicons_fontface").length) {
			$('head').append(g);
		}
	};

	IconExplorerSingleton.prototype.includeFontFace = function () {
		var self = this;
		var styleNode = document.createElement("style");
		styleNode.type = "text/css";
		styleNode.innerHTML = "@font-face { font-family: '" + self._custFont.name + "';" +
			"src: url('" + self._custFont.path + "');" +
			"}";
		document.head.appendChild(styleNode);
	};

	IconExplorerSingleton.prototype.bindEvents = function () {
		var self = this;
		$("#icon_explorer_content").on("click", ".icon-minified", { self: self }, function (event) {
			var self = event.data.self;
			var iconurl = $(this).data('value');
			var ele = self.boundElement;
			if (ele) {
				var eleId = ele.attr('id') || ele.attr('class');
				//ele.val(iconurl);
				$("#close_ix_panel").click();
				if (self.callBackStore[eleId]) {
					self.callBackStore[eleId](self.component, iconurl, eleId);
				}
			}
			//self.boundElement =null;
		});

		$("#close_ix_panel,button.close-modal").on("click", function () {
			$("body").removeClass("modal-open");
			$("div.icon-explorer-modal-aps").hide().remove("in");
			$("div.modal-backdrop").remove();
		});

		$("#icon_explorer_icontype").change(function () {
			if ($(this).val() == 'none') {
				var ele = self.boundElement;
				$("#close_ix_panel").click();
				if(ele){
					var eleId = ele.attr('id') || ele.attr('class');
					if (self.callBackStore[eleId]) {
						self.callBackStore[eleId](self.component, 'None', eleId);
					}
				}				
			}
			else {
				self.renderIconExplorer();
			}
		});

		$("#icon_explorer_input").on('keyup', function () {
			self.renderIconExplorer();
		});
	};

	IconExplorerSingleton.prototype.insertIconExplorerDom = function () {
		var iconExplorerModalDom = '<div class="modal fade icon-explorer-modal-aps" tabindex="-1" role="dialog"></div>';
		var iconExplorerDom = '<div class="modal-dialog modal-lg" role="document"> <div class="modal-content"> <div class="modal-header"><button type="button" id="close_ix_panel" class="close" aria-label="Close"><span aria-hidden="true">×</span></button> <h4 class="modal-title">Icon Explorer</h4> </div> <div class="modal-body"> <div class="icon_explorer_search row"> <div class="col-xs-6 col-md-9"> <div class="input-group"><span class="input-group-addon"><i class="fa fa-search" aria-hidden="true"></i></span><input id="icon_explorer_input" type="text" class="input form-control" placeholder="Search icons.."></div> </div> <div class="col-xs-6 col-md-3"> <div class="input-group-btn input-group-btn_vbi"><select id="icon_explorer_icontype" class="form-control"><option value="fa" selected>Font Awesome</option><option value="sap">SAP</option><option value="oth">Other</option><option value="custom">Custom</option><option value="none">None</option></select></div> </div> </div> <div id="icon_explorer_content" class="icon_content row"></div> </div> <div class="modal-footer"><button type="button" class="btn btn-default btn-sm close-modal" data-dismiss="modal">Cancel</button></div> </div> </div>';

		if ($("div.icon-explorer-modal-aps").length === 0) {
			$("body").append(iconExplorerModalDom);
			$("div.icon-explorer-modal-aps").append(iconExplorerDom);	
		}

	};



	IconExplorerSingleton.prototype.createIcon = function (prop) {
		var dom = [];

		if (prop.type == 'sap' || prop.type == 'custom') {
			var fontfamily = prop.type == 'sap' ? 'SAP-icons' : prop.name;
			dom.push('<span role="presentation" aria-hidden="true" aria-label="');
			dom.push(prop.c);
			/*dom.push('" title="');
			dom.push(prop.c);*/
			dom.push('" data-sap-ui-icon-content="');
			dom.push(prop.content);
			dom.push('" class="sapUiIcon sapUiIconMirrorInRTL" style="font-family:' + fontfamily + '";width:');
			dom.push((prop.width || '22px'));
			dom.push(';height:');
			dom.push((prop.height || '22px'));
			dom.push(';line-height:');
			dom.push((prop.lineHeight || '22px'));
			dom.push(';font-size:');
			dom.push((prop.fontsize || '22px'));
			dom.push('"></span>');
		}
		return dom.length ? dom.join('') : '';
	};



	IconExplorerSingleton.prototype.renderSapIcons = function (searchVal) {
		var self = this;
		var iconList = self.getIconList('sap');
		var fragDoc = document.createDocumentFragment();
		var iconContainer;
		var iconDom;
		var iconurl;
		var dataStore;
		if (searchVal === '') {
			$.each(iconList, function (i, v) {
				iconDom = self.createIcon({
					type: 'sap',
					content: String.fromCharCode(parseInt(v, 16)),
					c: i
				});
				iconurl = 'sap://' + i;
				dataStore = ' data-value=' + iconurl;
				iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '></div>');
				iconContainer.append(iconDom);
				fragDoc.appendChild(iconContainer[0]);
			});
		}
		else {
			var searchReg = new RegExp(searchVal, "i");
			$.each(iconList, function (i, v) {
				if (i.search(searchReg) != -1) {
					iconDom = self.createIcon({
						type: 'sap',
						content: String.fromCharCode(parseInt(v, 16)),
						c: i
					});
					iconurl = 'sap://' + i;
					dataStore = ' data-value=' + iconurl;
					iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '></div>');
					iconContainer.append(iconDom);
					fragDoc.appendChild(iconContainer[0]);
				}
			});
		}
		self.appendFragDoc(fragDoc);
	};

	IconExplorerSingleton.prototype.renderFontAwesomeIcons = function (searchVal) {
		var self = this;
		var iconList = self.getIconList('fa');
		var fragDoc = document.createDocumentFragment();
		var iconContainer;
		var icon;
		var iconurl;
		var dataStore;
		if (searchVal === '') {
			$.each(iconList, function (i, v) {
				icon = '<i class="fa ' + v + '"></i>';
				iconurl = 'fa://' + i;
				dataStore = ' data-value=' + iconurl;
				iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '>' + icon + '</div>');
				fragDoc.appendChild(iconContainer[0]);
			});
		}
		else {
			var searchReg = new RegExp(searchVal, "i");
			$.each(iconList, function (i, v) {
				if (i.search(searchReg) != -1) {
					icon = '<i class="fa ' + v + '"></i>';
					iconurl = 'fa://' + i;
					dataStore = ' data-value=' + iconurl;
					iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '>' + icon + '</div>');
					fragDoc.appendChild(iconContainer[0]);
				}
			});
		}
		self.appendFragDoc(fragDoc);
	};

	IconExplorerSingleton.prototype.renderOtherIcons = function (searchVal) {
		var self = this;
		var iconList = self.getIconList('oth');
		var fragDoc = document.createDocumentFragment();
		var iconContainer;
		var dataStore;
		var iconProps;
		var iconurl;
		if (searchVal === '') {
			$.each(iconList, function (i, v) {
				iconurl = 'oth://' + i;
				dataStore = ' data-value=' + iconurl;
				iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" style="width:26px;height:26px;padding:3px" title="' + iconurl + '"' + dataStore + '></div>');
				iconProps = {
					symbol: v,
					container: iconContainer
				};
				self.renderHarveyIcons(iconProps);
				fragDoc.appendChild(iconContainer[0]);
			});
		}
		else {
			var searchReg = new RegExp(searchVal, "i");
			$.each(iconList, function (i, v) {
				if (i.search(searchReg) != -1) {
					iconurl = 'oth://' + i;
					dataStore = ' data-value=' + iconurl;
					iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" style="width:26px;height:26px;padding:3px" title="' + iconurl + '"' + dataStore + '></div>');
					iconProps = {
						symbol: v,
						container: iconContainer
					};
					self.renderHarveyIcons(iconProps);
					fragDoc.appendChild(iconContainer[0]);
				}
			});
		}

		self.appendFragDoc(fragDoc);
	};


	IconExplorerSingleton.prototype.renderCustomIcons = function (searchVal) {
		var self = this;
		if (self._custFont && self._custFont.icons && Object.keys(self._custFont.icons).length) {
			var iconContainer;
			var iconJson = self._custFont.icons;
			var fragDoc = document.createDocumentFragment();
			var iconDom;
			var dataStore;
			var iconurl;
			if (searchVal === '') {
				$.each(iconJson, function (i, v) {
					iconDom = self.createIcon({
						type: 'custom',
						content: String.fromCharCode(parseInt(v, 16)),
						c: i,
						name: self._custFont.name
					});
					iconurl = 'custom://' + i;
					dataStore = ' data-value=' + iconurl;
					iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '></div>');
					iconContainer.append(iconDom);
					fragDoc.appendChild(iconContainer[0]);
				});
			}
			else {
				var searchReg = new RegExp(searchVal, "i");
				$.each(iconJson, function (i, v) {
					if (i.search(searchReg) != -1) {
						iconDom = self.createIcon({
							type: 'custom',
							content: String.fromCharCode(parseInt(v, 16)),
							c: i,
							name: self._custFont.name
						});
						iconurl = 'custom://' + i;
						dataStore = ' data-value=' + iconurl;
						iconContainer = $('<div id="ix_icon-' + i + '" class="icon-minified" title="' + iconurl + '"' + dataStore + '></div>');
						iconContainer.append(iconDom);
						fragDoc.appendChild(iconContainer[0]);
					}
				});
			}
			self.appendFragDoc(fragDoc);
		}
	};

	IconExplorerSingleton.prototype.parseFont = function (font, _custFont) {
		_custFont.icons = {};
		var aUnicodes = new Array(font.glyphs.length);
		var aNames = font.glyphNames.names;
		if (aNames.length === 0) {
			aNames = aUnicodes;
		}
		for (var i = 0; i < aUnicodes.length; i++) {
			aUnicodes[i] = IconExplorerSingleton.prototype.formatUnicode.call(undefined, font.glyphs[i].unicode);

			if (aNames[i] === undefined) { aNames[i] = aUnicodes[i]; }
			if (aNames[i] !== undefined && aNames[i] !== null) {
				_custFont.icons[aNames[i]] = aUnicodes[i];
			}
		}
	};

	IconExplorerSingleton.prototype.formatUnicode = function (unicode) {
		if (unicode !== undefined) {
			unicode = unicode.toString(16);
			if (unicode.length > 4) {
				return ("000000" + unicode.toUpperCase()).substr(-6);
			} else {
				return ("0000" + unicode.toUpperCase()).substr(-4);
			}
		} else { return; }
	};

	IconExplorerSingleton.prototype.processCustomIcons = function (searchVal) {
		var self = this;
		if (self.customFontProp) {
			self._custFont = "";
			try {
				self._custFont = self.component.callRuntimeHandler(self.customFontProp);
			}
			catch (e) {
			}

			if (self._custFont) {
				self._custFont = JSON.parse(self._custFont);
				self.addToIconPool('custom', self._custFont.icons);
				self.includeFontFace();
				self.renderCustomIcons(searchVal);
			}
			else {
				self.appendFragDoc();
			}
		}
		else {
			self.appendFragDoc();
		}
	};

	IconExplorerSingleton.prototype.appendFragDoc = function (fragDoc) {
		var iconContainer = $("#icon_explorer_content");
		iconContainer.empty();
		if (fragDoc) {
			iconContainer.append(fragDoc);
		}
	};

	IconExplorerSingleton.prototype.highlightIcon = function (pv) {
		if (pv) {
			$("[id='ix_icon-" + pv['key'] + "']").children().css({ "color": "#0199CD" });
		}
	};

	IconExplorerSingleton.prototype.renderIconExplorer = function (previousVal) {
		var self = this;
		var previousValObj = self.getIconInfo(previousVal);
		var renderFunc = {
			'fa': self.renderFontAwesomeIcons,
			'sap': self.renderSapIcons,
			'oth': self.renderOtherIcons,
			'custom': self.processCustomIcons
		};

		if (previousValObj) {
			$("#icon_explorer_icontype").val(previousValObj.type);
		}
		if(typeof self.iconExplorerType !== "undefined"){
			$("#icon_explorer_icontype").val(this.iconExplorerType).attr('disabled',true);
		}else if(self.iconExplorerType == undefined){ // For Location Analyser only fa and sap.
			$("#icon_explorer_icontype option[value='custom']").remove();
			$("#icon_explorer_icontype option[value='none']").remove();
			//Not for AdvancedKPITile
			if (self.component.sid !== "ADVANCEDKPITILE") {
				$("#icon_explorer_icontype option[value='oth']").remove();
			}
		}

		//var iconContainer = $("#icon_explorer_content");
		var iconType = $("#icon_explorer_icontype").val();
		var searchVal = $("#icon_explorer_input").val();

		if (renderFunc[iconType]) {
			if (!searchVal) {
				searchVal = '';
			}
			renderFunc[iconType].call(self, searchVal);
			try {
				self.highlightIcon(previousValObj);
			}
			catch (e) {
				//alert("error : " + e);
			}

		}
	};

	IconExplorerSingleton.prototype.renderHarveyIcons = function (iconProps) {

		var container = iconProps.container;

		var iconSymbol = iconProps.symbol;

		var harveyJson = {
			'harveyzerodeg': '<svg  class="harvey-svg" viewBox="0 0 239.09 239.09"><circle class="hvy0_cls-1" cx="119.54" cy="119.54" r="114.54" style=""/></svg>',

			'harvey25deg': '<svg class="harvey-svg" viewBox="0 0 239.09 239.09"><path d="M190.5,87.08A114.54,114.54,0,0,1,305,201.62H190.5" transform="translate(-70.96 -82.08)" class="hvy25_cls-1" /><circle cx="119.54" cy="119.54" r="114.54" class="hvy25_cls-2" /></svg>',

			'harvey50deg': '<svg class="harvey-svg" viewBox="0 0 239.09 239.09"><circle cx="119.54" cy="119.54" r="114.54" class="hvy50_cls-1" /><path d="M190.5,87.08a114.54,114.54,0,0,1,0,229.09" transform="translate(-70.96 -82.08)" class="hvy50_cls-2" /></svg>',

			'harvey75deg': '<svg class="harvey-svg" viewBox="0 0 249.09 249.09"><circle class="hvy75_cls-1" cx="124.55" cy="124.55" r="119.55" /><path class="hvy75_cls-2" d="M408.5,149V268.5H289A119.55,119.55,0,1,0,408.5,149Z" transform="translate(-283.95 -143.95)"/></svg>',

			'harvey100deg': '<svg class="harvey-svg" viewBox="0 0 239.09 239.09"><circle class="hvy100_cls-1" cx="119.54" cy="119.54" r="114.54" /></svg>'
		};

		if (harveyJson[iconSymbol]) {
			container.append(harveyJson[iconSymbol]);
		}
	};
	
	return IconExplorerSingleton;

}());