/*globals familyDrilldown, validateImplementMethod */
/*jshint unused:false*/
function familyDrilldown(args) {
    var that = this;
    /*
     * *********************************************************************
     * -Template for defining definition in familySeries
     * -All function definition need to be within the scope of familyXYZ
     * -Each definition of property should be based on the validation of
     *  method validateImplementMethod(args, [[function name string]])
     *
     *  Example:
     *  if (validateImplementMethod(args, 'cds')) {
     *    <-- Actual Function definition goes here -->
     *    that.cds = function() {
     *
     *      // Your own implementation starts here either extend or
     *      // write complete new implementation
     *
     *      // var result = SeriesLibrary.prototype.cds.call(that);
     *      // some specific implementation
     *
     *    };
     *  }
     *
     * -If function definition is defined without validateImplementMethod
     *  then particular method is included by default in all components that
     *  include this file!
     * *********************************************************************
     */
    //   alert('[[Executed familyDrilldown]]');
    if (validateImplementMethod(args, 'cds')) {
        that.cds = function() {
            window.alert('function cds "hasOwnProperty" accessed');
        };
    }

    /**
   * Get bardatacustomization object which is compatible with previous version. It returns only those dimensions whose 'show' property is set to 'true'
   * @param {object} component - Reference to component object
   * @returns {object} 
   */
    that.getBarDataCustomization = function(component){
        var barData = component.bardatacustomization();
        var dataSourceDefinition = typeof component.dataSourceDefinition !== 'undefined' ? component.dataSourceDefinition() : 'technicalId';
        var resultSet = component.result();
        var duConfig = {
            COLUMNS: [],
            ROWS: []
        };

        if(barData !== '' && barData !== undefined && barData !== null){
          try{
            barData = JSON.parse(barData);
          }catch(e){}
        }

        if(resultSet !== '' && resultSet !== undefined && resultSet !== null){
          try{
            resultSet = JSON.parse(resultSet);
          }catch(e){}
        }


        if(barData && typeof barData === 'object'){
          if(dataSourceDefinition === 'technicalId'){
            _.each(duConfig, function(config, key){
              _.each(barData[key], function(dimension){
                if(dimension.show === true || typeof dimension.show === 'undefined'){
                      config.push(dimension);
                }
              });
            });
          }
          else if(dataSourceDefinition === 'index' && resultSet){
            _.each(duConfig, function(config, key){
              _.each(barData[key], function(index){
                index = parseInt(index, 10);
                if(!_.isNaN(index)){
                  var dimension = resultSet.dimensions[index];
                  config.push({
                    key: dimension.key,
                    text: dimension.text,
                    index: index,
                    show: true
                  });
                }
              });
            });
          }
        }
        return duConfig;    
    };

    if (validateImplementMethod(args, 'createDrilldownMultilevelData')) {
        that.createDrilldownMultilevelData = function(component, selectedDimensions, selectedMeasure, type/*, colorSeries*/) {
            var self = this;
            var measureMembers = component.dataRes.columnCount,
                tupleStringArray = [],
                //DrilldownData = [],
                SeriesData = [{}],
                currentTuplePos = -1,
                tupleStringArr = [],
                tupleString = '',
                tempMemberCollection = {},
                pos = 0;
            var tempMemberCollectionDataIndex = {};
            SeriesData[0].backButton = component.dataRes.dimensions[selectedDimensions[0]]["picker_text"];
            SeriesData[0].data = [];
            var seriesDataJson = {};
            var tempSeriesKeyIndexes = [];
            var tempSeriesKeyNames = [];
            var tempSeriesNames = [];
            var curTuple = component.dataRes.tuples[selectedMeasure];
            var noOfColumns = self.getNumberOfRowsColumns(component, 'COLUMNS');
            var tempZtlColJson = {};
            var unitOfMeasure = '';
            var scalingFactor = 0;
            var measureDimIndex = '';
			var showtotaldatabase = component.showtotaldatabase();
            var hasResultIntupleCol = [];
            var indexInfo;
            var indexInfoArray;
            
            /**
             * VBX141-7796 
             * Making series consistent with other bar family charts where Measure Dimension is always taken first while making series name
             */
            var selectedColumndimIndexes = [];
            var measureDimensionIndex = 0;
            for (var i = 0; i < noOfColumns; i++) {
                if (component.dataRes.dimensions[i].containsMeasures === true) {
                    selectedColumndimIndexes.unshift(i);
                    measureDimensionIndex = i;
                }
                else{
                    selectedColumndimIndexes.push(i);
                }
            }

            for (var i=0, j = 0; j < selectedColumndimIndexes.length; j++) {
                i = selectedColumndimIndexes[j];
                if (i === measureDimensionIndex) {
                    measureDimIndex = i + ':' + curTuple[i];
                    if (component.unitcurrency()) {
                        unitOfMeasure = component.dataRes.dimensions[i].members[curTuple[i]].unitOfMeasure || '';
                    }

                    if (component.scalingfactorenable()) {
                        scalingFactor = component.dataRes.dimensions[i].members[curTuple[i]].scalingFactor || 0;
                    }
                }

                if (curTuple[i] !== -1) {
                    tempSeriesKeyIndexes.push(i + ':' + curTuple[i]);
                    tempSeriesKeyNames.push(component.dataRes.dimensions[i].members[curTuple[i]].key);
                    tempSeriesNames.push(component.dataRes.dimensions[i].members[curTuple[i]].text);
                    tempZtlColJson[component.dataRes.dimensions[i].key] = {
                        'internalKey': component.dataRes.dimensions[i].members[curTuple[i]].key,
                        'text': component.dataRes.dimensions[i].members[curTuple[i]]["picker_text"]
                    };
                }
            }
			
			/*#VBX141-4729*/
			if(showtotaldatabase){				
				for(var j=0, jLen = noOfColumns; j<jLen; j++){
					if(component.dataRes.dimensions[j].members[curTuple[j]].type == "RESULT"){					  
						hasResultIntupleCol.push(j);
					}
				}
			}
            tempSeriesNames = _.filter(tempSeriesNames, function(x) {              
              return (x != '' && x!= null && x!=" ");
            });
            var seriesKeyIndexes = tempSeriesKeyIndexes.join('-');
            var seriesKeyNames = tempSeriesKeyNames.join('-');
            var seriesNames = tempSeriesNames.join('-');
            if (seriesKeyIndexes === '') {
                seriesKeyIndexes = 'none:none';
                seriesKeyNames = 'Series-1';
                seriesNames = 'Series-1';
            }

            component.ztlSelectedColumnJson[seriesKeyNames] = $.extend(true, {}, tempZtlColJson);
            SeriesData[0].columnIndexKey = seriesKeyIndexes;
            SeriesData[0].id = 'Series-' + seriesKeyIndexes;
            SeriesData[0].seriesname = seriesNames;
            SeriesData[0].seriesKeyName = seriesKeyNames;
            SeriesData[0].unitOfMeasure = unitOfMeasure;
            SeriesData[0].scalingFactor = Math.pow(10, scalingFactor);
            var scalingText = component.scalingfactorenable() ? '(*' + SeriesData[0].scalingFactor + ')' : '';
            if (type == 'bardd') {
                /*var temp_unit_measure='';
        if(SeriesData[0].unitOfMeasure){
			temp_unit_measure="(in " + SeriesData[0].unitOfMeasure + ")";
		}*/
                SeriesData[0].name = self.formatSeriesName(component, seriesNames, measureDimIndex, {
                    'showunit': 'unitcurrency'
                });
            }		
			

            var totalForChart = [];
			var seriesDataJsonWithResult = {};
			var tempMemberCollectionWithResult = {};
            for (var selDim = 0; selDim < selectedDimensions.length; selDim++) {
                for (var j = selectedMeasure; j < component.dataRes.data.length; j += measureMembers) {
                    tupleString = '';
                    tupleStringArr = [];
                    indexInfo = '';
                    indexInfoArray = [];
					var tempTuple = component.dataRes.tuples[j];
                    //var containResult = false;
                    for (var i = 0; i <= selDim; i++) {
                        indexInfoArray.push(selectedDimensions[i] + ':' + tempTuple[selectedDimensions[i]]);
                        tupleStringArr.push(component.dataRes.dimensions[selectedDimensions[i]].key.replace(/-/g, '_') + ':' + component.dataRes.dimensions[selectedDimensions[i]].members[component.dataRes.tuples[j][selectedDimensions[i]]].key.replace(/-/g, '_'));
                        //console.log("selectedDimensions[i] = " +selectedDimensions[i] + " : j= " + j);
                        //dimMemkeyArr.push(component.dataRes.dimensions[selectedDimensions[i]].key.replace(/-/g,'_') + ':' + component.dataRes.dimensions[selectedDimensions[i]].members[component.dataRes.tuples[j][selectedDimensions[i]]].key.replace(/-/g,'_'));
                    }

                    if (self.vbiCharts.eliminateResult(component.dataRes, tempTuple, undefined, component.showtotaldatabase()) === true) {
                        tupleString = tupleStringArr.join('-');
                        indexInfo = indexInfoArray.join('-');
                        currentTuplePos = tupleStringArray.indexOf(tupleString);
						var hasResultIntuple = [];						
						if(showtotaldatabase){
							for(var k=noOfColumns, kLen = tempTuple.length; k< kLen; k++){
								if(component.dataRes.dimensions[k].members[tempTuple[k]].type == "RESULT"){
									hasResultIntuple.push(tempTuple[k]);
								}
							}
						}
						
                        if (selDim === 0) {
                            if (currentTuplePos == -1) {
                                tupleStringArray.push(tupleString);
                                if (!seriesDataJson[tupleString]) {
                                    seriesDataJson[tupleString] = {};
                                    seriesDataJson[tupleString].drilldown = tupleString;
                                    seriesDataJson[tupleString].Namekey = tupleString;
                                    seriesDataJson[tupleString].columnKey = seriesKeyNames;
                                    seriesDataJson[tupleString].columnIndexKey = seriesKeyIndexes;
                                    seriesDataJson[tupleString].indexInfo = indexInfo;
                                    if (typeof component.ztlSelectedRowJson[tupleString] == 'undefined') {
                                        var dimKey = component.dataRes.dimensions[selectedDimensions[selDim]].key;
                                        component.ztlSelectedRowJson[tupleString] = {};
                                        component.ztlSelectedRowJson[tupleString][dimKey] = {
                                            internalKey: component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]].key,
                                            text: component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]]["picker_text"]
                                        };
                                    }

                                    seriesDataJson[tupleString].nodeState = 'collapsed';
                                    seriesDataJson[tupleString].level = 0;
                                    seriesDataJson[tupleString].parentKey = 'none';
                                    //seriesDataJson[tupleString].color = colorSeries[tupleString]||colorSeries[0];
                                    seriesDataJson[tupleString].name = component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]].text;
                                    seriesDataJson[tupleString]['yArr'] = [component.dataRes.data[j]];
                                    seriesDataJson[tupleString].y = 0;
                                }
                            } else {
                                seriesDataJson[tupleString]['yArr'].push(component.dataRes.data[j]);
                            }
							
							
							/*#VBX141-4729*/
							if(typeof seriesDataJsonWithResult[tupleString] === "undefined"){
								seriesDataJsonWithResult[tupleString] = {"yArr" : []};
							}
							
							if(showtotaldatabase && (hasResultIntuple.length || hasResultIntupleCol.length)){	
								var lastTempStore, lastTempstoreLen = 0;
								var totalLen = hasResultIntupleCol.length + hasResultIntuple.length;
								if(seriesDataJsonWithResult[tupleString]["yArr"].length){
									lastTempStore = seriesDataJsonWithResult[tupleString]["yArr"][seriesDataJsonWithResult[tupleString]["yArr"].length - 1];
									lastTempstoreLen = lastTempStore[0];
								}
								if(lastTempstoreLen < totalLen){
									seriesDataJsonWithResult[tupleString]["yArr"].length = 0;						
								}					
								seriesDataJsonWithResult[tupleString]["yArr"].push([totalLen , (component.dataRes.data[j] === null ? 0 : component.dataRes.data[j])]);
							}				
							

                            totalForChart.push(component.dataRes.data[j]);
                        } else {
                            var tsKey = tupleString.slice(0, tupleString.lastIndexOf('-'));
                            if (currentTuplePos == -1) {
                                tupleStringArray.push(tupleString);
                                if (!tempMemberCollection[tsKey]) {
                                    tempMemberCollection[tsKey] = {};
                                    tempMemberCollectionDataIndex[tsKey] = {};
                                    //tempMemberCollection[tsKey].id = tupleString;
                                    tempMemberCollection[tsKey].columnIndexKey = seriesKeyIndexes;
                                    tempMemberCollection[tsKey].seriesname = seriesNames;
                                    tempMemberCollection[tsKey].unitOfMeasure = unitOfMeasure;
                                    tempMemberCollection[tsKey].scalingFactor = Math.pow(10, scalingFactor);
                                    tempMemberCollection[tsKey].backButton = component.dataRes.dimensions[selectedDimensions[selDim]]["picker_text"];
                                    scalingText = component.scalingfactorenable() ? '(*' + tempMemberCollection[tsKey].scalingFactor + ')' : '';
                                    if (type == 'bardd') {
                                        tempMemberCollection[tsKey].name = SeriesData[0].name;
                                        /*if(tempMemberCollection[tsKey].name.indexOf('(* ') == -1)
                                        {
                                              tempMemberCollection[tsKey].name = seriesNames + ' ' + scalingText;
                                        }

                                        if(tempMemberCollection[tsKey].unitOfMeasure)
                                        {
                                            tempMemberCollection[tsKey].name += " (in " + tempMemberCollection[tsKey].unitOfMeasure + ")";
                                        }*/

                                    }

                                    tempMemberCollection[tsKey].seriesKeyName = seriesKeyNames;
                                    tempMemberCollection[tsKey].value = 0;
									tempMemberCollection[tsKey].valArr = [];
                                    // need attention!!!
                                    tempMemberCollection[tsKey].data = [];
                                }

                                var parentKey = tsKey;
                                tempMemberCollectionDataIndex[tsKey][tupleString] = tempMemberCollection[tsKey].data.length;
                                tempMemberCollection[tsKey].data.push({
                                    'drilldown': selDim < selectedDimensions.length - 1 ? true : false,
                                    'name': component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]].text,
                                    'Namekey': tupleString,
                                    'indexInfo': indexInfo,
                                    'parentKey': parentKey,
                                    'level': parentKey.split('-').length,
                                    'nodeState': 'collapsed',
                                    'columnKey': seriesKeyNames,
                                    'columnIndexKey': seriesKeyIndexes,
                                    'y': 0,
                                    'yArr': [component.dataRes.data[j]]
                                });
                                if (typeof component.ztlSelectedRowJson[tupleString] == 'undefined') {
                                    var dimKey = component.dataRes.dimensions[selectedDimensions[selDim]].key;
                                    component.ztlSelectedRowJson[tupleString] = {};
                                    component.ztlSelectedRowJson[tupleString][dimKey] = {
                                        internalKey: component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]].key,
                                        text: component.dataRes.dimensions[selectedDimensions[selDim]].members[component.dataRes.tuples[j][selectedDimensions[selDim]]]["picker_text"]
                                    };
                                }

                                //tempMemberCollection[tsKey].value += component.dataRes.data[j];
                            } else {
                                pos = tempMemberCollectionDataIndex[tsKey][tupleString];
                                tempMemberCollection[tsKey].data[pos]['yArr'].push(component.dataRes.data[j]);
                                //tempMemberCollection[tsKey].value += component.dataRes.data[j];
                            }
							tempMemberCollection[tsKey].valArr.push(component.dataRes.data[j]);
							
								try{
								/*#VBX141-4729*/
								if(typeof tempMemberCollectionWithResult[tsKey] === "undefined"){
									tempMemberCollectionWithResult[tsKey] = {"valArr" : [], "data" : []};
								}
								pos = tempMemberCollectionDataIndex[tsKey][tupleString];
								tempMemberCollectionWithResult[tsKey].data[pos] = [];

								if(showtotaldatabase && (hasResultIntuple.length || hasResultIntupleCol.length)){	
									var lastTempStore, lastTempstoreLen = 0;
									var totalLen = hasResultIntupleCol.length + hasResultIntuple.length;

									// do for value
									if(tempMemberCollectionWithResult[tsKey].valArr.length){
										lastTempStore = tempMemberCollectionWithResult[tsKey].valArr[tempMemberCollectionWithResult[tsKey].valArr.length - 1];
										lastTempstoreLen = Number(lastTempStore[0]);
									}
									if(lastTempstoreLen < totalLen){
										tempMemberCollectionWithResult[tsKey].valArr.length = 0;						
									}										
									tempMemberCollectionWithResult[tsKey].valArr.push([totalLen ,(component.dataRes.data[j] === null ? 0 : component.dataRes.data[j])]);

									// do for data
									lastTempstoreLen = 0;
									 
									if(typeof tempMemberCollectionWithResult[tsKey].data[pos] === "undefined"){
										tempMemberCollectionWithResult[tsKey].data[pos] = [];
									}
									if(tempMemberCollectionWithResult[tsKey].data[pos].length){
										lastTempStore = tempMemberCollectionWithResult[tsKey].data[pos][tempMemberCollectionWithResult[tsKey].data[pos].length - 1];
										lastTempstoreLen = Number(lastTempStore[0]);
									}
									if(lastTempstoreLen < totalLen){
										tempMemberCollectionWithResult[tsKey].data[pos].length = 0;						
									}										
									tempMemberCollectionWithResult[tsKey].data[pos].push([totalLen , (component.dataRes.data[j] === null ? 0 : component.dataRes.data[j])]);
								}
							}catch(e){
								console.log(e); // jshint ignore:line
							}						
                        }
                    }
                }
            }
			
			
			if(showtotaldatabase){
				$.each(seriesDataJsonWithResult, function(tupleString, val){
					if(val.yArr.length){
						seriesDataJson[tupleString]['yArr'].length = 0;
						for(var i=0, iLen = val.yArr.length; i<iLen; i++){
							seriesDataJson[tupleString]['yArr'].push(Number(val.yArr[i][1]));
						}
					}
				});
				
				$.each(tempMemberCollectionWithResult, function(tsKey, val){
					if(val.valArr.length){
						tempMemberCollection[tsKey].valArr.length = 0;
						for(var i=0, iLen = val.valArr.length; i<iLen; i++){
							tempMemberCollection[tsKey].valArr.push(Number(val.valArr[i][1]));
						}
					}
					
					
					$.each(val.data, function(index, d){
						if(d.length){
							tempMemberCollection[tsKey].data[index].yArr.length = 0;
							for(var i=0, iLen = d.length; i<iLen; i++){
								tempMemberCollection[tsKey].data[index].yArr.push(Number(d[i][1]));
							}
						}
					});
					
				});
			}
			
            for (var x in tempMemberCollection) {
                //DrilldownData.push(tempMemberCollection[x]);
                $.each(tempMemberCollection[x].data, function(i, v) {
                    v.y = component.dataUtilityAggregation(v['yArr']);
                    delete v['yArr'];
                });
				
				tempMemberCollection[x].value = component.dataUtilityAggregation(tempMemberCollection[x]['valArr']);
				delete tempMemberCollection[x]['valArr'];

                component.drilldownjson[x + '-' + seriesKeyNames] = tempMemberCollection[x];
            }

            for (var x in seriesDataJson) {
                seriesDataJson[x].y = component.dataUtilityAggregation(seriesDataJson[x]['yArr']);
                delete seriesDataJson[x]['yArr'];
                SeriesData[0].data.push(seriesDataJson[x]);
            }

            if (component.showtotals()) {
                var tempTotal = {};
                tempTotal.drilldown = false;
                tempTotal.Namekey = 'total';
                tempTotal.indexInfo = "-1:-1"; // index not available in resultset
                tempTotal.columnKey = seriesKeyNames;
                tempTotal.columnIndexKey = seriesKeyIndexes;
                tempTotal.nodeState = 'collapsed';
                tempTotal.level = 0;
                tempTotal.parentKey = 'none';
                tempTotal.name = 'Total';
                tempTotal.y = component.dataUtilityAggregation(totalForChart);
                SeriesData[0].data[component.totalnodeplacement() == 'before' ? 'unshift' : 'push'](tempTotal);
            }
            //seriesDataJson[tupleString].color = colorSeries[tupleString]||colorSeries[0];
            tempMemberCollection = null;
            seriesDataJson = null;
            return [SeriesData /*,DrilldownData*/ ];
        };
    }

    /*if (validateImplementMethod(args, 'createMembersPieDataDrilldown')) {
        that.createMembersPieDataDrilldown = function(component, categoriesIndex, selectedPoints) {
            component.memberDatasCollection = [];
            var vbiCharts = new VbiCharts(component);
            var seriesLibrary = new SeriesLibrary(vbiCharts);
            $.each(selectedPoints, function(i, currentpoint) {
                component.memberDatasCollection.push(seriesLibrary.createMemberDataDrilldown(component, categoriesIndex[currentpoint.name], columnSelected));
            });

            return component.memberDatasCollection;
        };
    }*/

    if (validateImplementMethod(args, 'createMemberDataDrilldown')) {
        that.createMemberDataDrilldown = function(component, categoryName, serieskey) {
            component.memberDataCollection = {};
            var catArr = {};
            if (component.drilldownMemberjson.hasOwnProperty(categoryName)) {
                catArr = component.drilldownMemberjson[categoryName];
                component.memberDataCollection[catArr.dimension] = catArr;
            }

            catArr = [];
            if (serieskey !== undefined) {
                catArr = catArr.concat(serieskey.split('-'));
                for (var i = 0; i < catArr.length; i++) {
                    var dimMemIndex = catArr[i].split(':');
                    component.memberDataCollection[component.dataRes.dimensions[dimMemIndex[0]].key] = {
                        'internalKey': component.dataRes.dimensions[dimMemIndex[0]].members[dimMemIndex[1]].key,
                        'text': component.dataRes.dimensions[dimMemIndex[0]].members[dimMemIndex[1]]["picker_text"]
                    };
                }
            }

            return JSON.stringify(component.memberDataCollection);
        };
    }
    // ****Component.JS
    if (validateImplementMethod(args, 'createHierarchySeriesData')) {
        that.createHierarchySeriesData = function(component, type, defaultValue) {
            //      alert('override')
            var self = this;
            //var count = 0;
            var defaultColor = [
                '#B586DA',
                '#FF6657',
                '#F3A730',
                '#946B66',
                '#B0AD53',
                '#7AC1DE',
                '#F1C337',
                '#566C6C',
                '#A1CC54',
                '#543F1E'
            ];
            var seriesColor = [];
            if (component.dataseriescolor() === '' || component.dataseriescolor() === undefined) {
                seriesColor = [];
            } else {
                seriesColor = seriesColor.concat(component.dataseriescolor().split(','));
            }

            component.drilldownMemberjson = {};
            //var ColumnDimensionCount = component.dataRes['axis_columns'][0].indexOf(-1);
            var selectedMember = 0;
            var sortedarray = [];
            var MemberText;
            var MemberKey;
            var rootNodes = [];
            var categories = [];
            var categoriesKey = [];
            var noOfColumns = self.getNumberOfRowsColumns(component, 'COLUMNS'); //component.dataRes['axis_columns'][0].indexOf(-1);
            var seriesNames;
			var seriesKeyNames;
            var seriesKeyIndexes;
            var seriesData = [];
            var measureColorPair = {};
            var selectedColumndimIndexes = [];
            if (!(component.measurecolorpair() === undefined || component.measurecolorpair() === '')) {
                measureColorPair = JSON.parse(component.measurecolorpair());
            }

            component.categoriesIndex = [];
            component.selectedColumnIndex = 0;
            component.selectedDimension = noOfColumns;
            component.nodeAlignment = 'top';
            if (component.enablehierarchydrilldown() === true) {
                for (var i = 0; i < component.dataRes.dimensions.length; i++) {
                    if (component.dataRes.dimensions[i].hasOwnProperty('hierarchy')) {
                        component.selectedDimension = i;
                        component.nodeAlignment = component.dataRes.dimensions[i].hierarchy.nodeAlignment;
                        break;
                    }
                }
            }

             /**
             * VBX141-7796 
             * Making series consistent with other bar family charts where Measure Dimension is always taken first while making series name
             */
            var selectedColumndimIndexes = [];
            var measureDimensionIndex = 0;
            for (var i = 0; i < noOfColumns; i++) {
                if (component.dataRes.dimensions[i].containsMeasures === true) {
                    selectedColumndimIndexes.unshift(i);
                    measureDimensionIndex = i;
                }
                else{
                    selectedColumndimIndexes.push(i);
                }
            }


            if (component.dataRes.dimensions[component.selectedDimension].axis == 'COLUMNS') {
                return 4;
            }

            function createSeriesData(k, seriesKeyNames, tempSeriesKeyIndexes) {
                sortedarray = [];
                selectedMember = 0;
                var totalForChart = 0;
                var isBottom = component.nodeAlignment === 'bottom' ? true : false;
                var len = component.dataRes.dimensions[component.selectedDimension].members.length;
                
                for (var i = isBottom ? len-1 : 0; (isBottom ? i >= 0 : i<len) ; isBottom ? i-- : i++) {
                    if (component.showtotaldatabase() || component.dataRes.dimensions[component.selectedDimension].members[i].type != 'RESULT') {
                        if (typeof component.dataRes.dimensions[component.selectedDimension].members[i].level == 'undefined' && rootNodes.indexOf(component.dataRes.dimensions[component.selectedDimension].members[i].key) == -1) {
                            rootNodes.push(component.dataRes.dimensions[component.selectedDimension].members[i].key);
                        }

                        if (component.includerootnodes() === true && typeof component.dataRes.dimensions[component.selectedDimension].members[i].level == 'undefined' || component.includerootnodes() === false && typeof component.dataRes.dimensions[component.selectedDimension].members[i].level != 'undefined' && component.dataRes.dimensions[component.selectedDimension].members[i].level == 1) {
                            MemberText = component.dataRes.dimensions[component.selectedDimension].members[i].text;
                            MemberKey = component.dataRes.dimensions[component.selectedDimension].members[i].key;
                            sortedarray[selectedMember] = {};
                            if (MemberText.indexOf('|') != -1) {
                                var newTempCatName = MemberText.replace(/\|/g, ' ');
                                //component.categoriesIndex[newTempCatName] = MemberText;
                                MemberText = newTempCatName;
                            } else {}

                            sortedarray[selectedMember].name = MemberText;
                            sortedarray[selectedMember].Namekey = component.dataRes.dimensions[component.selectedDimension].members[i].originalKey || component.dataRes.dimensions[component.selectedDimension].members[i].key;
                            sortedarray[selectedMember].indexInfo = component.selectedDimension + ':' + i;
                            if (typeof component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey] == 'undefined') {
                                var dimKey = component.dataRes.dimensions[component.selectedDimension].key;
                                component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey] = {};
                                component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey][dimKey] = {
                                    internalKey: component.dataRes.dimensions[component.selectedDimension].members[i].originalKey || component.dataRes.dimensions[component.selectedDimension].members[i].key,
                                    text: component.dataRes.dimensions[component.selectedDimension].members[i]["picker_text"]
                                };
                            }

                            if (type != 'bardd') {
                                sortedarray[selectedMember].color = measureColorPair[sortedarray[selectedMember].Namekey] ? measureColorPair[sortedarray[selectedMember].Namekey].color : self.getSeriesColors(component, seriesColor, selectedMember, defaultColor);
                            }

                            sortedarray[selectedMember].columnKey = seriesKeyNames;
                            sortedarray[selectedMember].columnIndexKey = tempSeriesKeyIndexes;
                            sortedarray[selectedMember].parentKey = 'none';
                            sortedarray[selectedMember].nodeState = 'collapsed';
                            sortedarray[selectedMember].level = typeof component.dataRes.dimensions[component.selectedDimension].members[i].level == 'undefined' ? 0 : 1;
                            if (categoriesKey.indexOf(MemberKey) == -1) {
                                categories.push(MemberText);
								//categoriesKey.push(MemberKey);
                            }
							
							
                            sortedarray[selectedMember].drilldown = component.dataRes.dimensions[component.selectedDimension].members[i].type == 'RESULT'? false : true;
                            sortedarray[selectedMember].y = [];
                            for (var j = k; j < component.dataRes.data.length; j += component.dataRes.columnCount) {
                                if (i == component.dataRes.tuples[j][component.selectedDimension] && self.vbiCharts.eliminateResult(component.dataRes, component.dataRes.tuples[j], undefined, component.showtotaldatabase()) === true) {
                                    sortedarray[selectedMember].y.push(component.dataRes.data[j]);
                                }
                            }

                            sortedarray[selectedMember].y = component.dataUtilityAggregation(sortedarray[selectedMember].y);
                            totalForChart += sortedarray[selectedMember].y;
                            selectedMember++;
                        }
                    }
                }

                if (component.showtotals()) {
                    var tempTotal = {};
                    tempTotal.name = 'Total';
                    tempTotal.Namekey = 'total';
                    tempTotal.columnKey = seriesKeyNames;
                    if (type != 'bardd') {
                        tempTotal.color = measureColorPair[tempTotal.Namekey] ? measureColorPair[tempTotal.Namekey].color : self.getSeriesColors(component, seriesColor, selectedMember, defaultColor);
                    }

                    tempTotal.columnIndexKey = tempSeriesKeyIndexes;
                    tempTotal.parentKey = 'none';
                    tempTotal.nodeState = 'collapsed';
                    tempTotal.level = typeof component.dataRes.dimensions[component.selectedDimension].members[0].level == 'undefined' ? 0 : 1;
                    tempTotal.drilldown = false;
                    tempTotal.y = totalForChart;
                    sortedarray[component.totalnodeplacement() == 'before' ? 'unshift' : 'push'](tempTotal);
                    /* jshint expr: true */ 
					categories.indexOf('Total') == -1 ? categories[component.totalnodeplacement() == 'before' ? 'unshift' : 'push']('Total') : ''; /* jshint expr: true */
                }

                return sortedarray;
            }

            if (component.datautility()/*self.globalDataUtility*/) { // check is needed to show smoke screen 
                return 5;
            } else if (type == 'bardd') {
                seriesData = [];
                component.ztlSelectedColumnJson = {};
                component.ztlSelectedRowJson = {};
                for (var k = 0; k < component.dataRes.columnCount; k++) {
                    var curTuple = component.dataRes.tuples[k];
                    var tempSeriesNames = [];
                    var tempSeriesKeys = [];
                    var tempSeriesKeyIndexes = [];
                    var tempZtlJson = {};
                    var unitOfMeasure = '';
                    var scalingFactor = 0;
                    //var scalingText = '';
                    var legendName = '';
                    var measureDimIndex = '';
                    for (var i = 0, j=0; i < selectedColumndimIndexes.length; i++) {
                        j = selectedColumndimIndexes[i];
                        if (component.showtotaldatabase() || component.dataRes.dimensions[j].members[curTuple[j]].type != 'RESULT') {
                            if (j === measureDimensionIndex) {
                                measureDimIndex = j + ':' + curTuple[j];
                                if (component.unitcurrency()) {
                                    unitOfMeasure = component.dataRes.dimensions[j].members[curTuple[j]].unitOfMeasure || '';
                                }

                                if (component.scalingfactorenable()) {
                                    scalingFactor = component.dataRes.dimensions[j].members[curTuple[j]].scalingFactor || 0;
                                }
                            }

                            tempSeriesNames.push(component.dataRes.dimensions[j].members[curTuple[j]].text);
                            tempSeriesKeys.push(component.dataRes.dimensions[j].members[curTuple[j]].key);
                            tempSeriesKeyIndexes.push(j + ':' + curTuple[j]);
                            tempZtlJson[component.dataRes.dimensions[j].key] = {
                                internalKey: component.dataRes.dimensions[j].members[curTuple[j]].key,
                                text: component.dataRes.dimensions[j].members[curTuple[j]]["picker_text"]
                            };
                        } else {
                            tempSeriesNames = [];
                            tempSeriesKeys = [];
                            tempSeriesKeyIndexes = [];
                            tempZtlJson = {};
                            break;
                        }
                    }


                    /**
                     * VBX141-4226
                     * Measure in background filter
                     */
                    if (selectedColumndimIndexes.length === 0 || tempSeriesNames.length !== 0) {
                        tempSeriesNames = _.filter(tempSeriesNames, function(x) {              
                          return (x != '' && x!= null && x!=" ");
                        });
                        seriesNames = tempSeriesNames.join('-');
                        seriesKeyNames = tempSeriesKeys.join('-');
                        seriesKeyIndexes = tempSeriesKeyIndexes.join('-');
                        component.ztlSelectedColumnJson[seriesKeyNames] = $.extend(true, {}, tempZtlJson);

                        if (seriesKeyNames === '') {
                            seriesKeyIndexes = 'none:none';
                            seriesKeyNames = 'Series-1';
                            seriesNames = 'Series-1';
                        }

                        legendName = seriesNames;
                        if (measureDimIndex) {
                            legendName = self.formatSeriesName(component, seriesNames, measureDimIndex, {
                                'showunit': 'unitcurrency'
                            });
                        }
                        /*
                        scalingText = component.scalingfactorenable() ? '(*' + Math.pow(10, scalingFactor) + ')' : '';
                        if(legendName.indexOf('(*') == -1)
                        {
                            legendName += ' ' + scalingText;
                        }
                        if(unitOfMeasure)
                        {
                            legendName +=  ' (in ' + unitOfMeasure+ ")";
                        }*/

                        seriesData.push({
                            name: legendName,
                            backButton: component.dataRes.dimensions[component.selectedDimension]["picker_text"],
                            id: 'Series-' + seriesKeyIndexes,
                            seriesname: seriesNames,
                            seriesKeyName: seriesKeyNames,
                            unitOfMeasure: unitOfMeasure,
                            scalingFactor: Math.pow(10, scalingFactor),
                            data: createSeriesData(k, seriesKeyNames, seriesKeyIndexes),
                            color: measureColorPair[seriesKeyNames] ? measureColorPair[seriesKeyNames].color : self.getSeriesColors(component, seriesColor, seriesData.length, defaultColor),
                            columnIndexKey: seriesKeyIndexes
                        });
                    }
                }

            }

            defaultValue.series = seriesData;

            defaultValue.xAxis.categories = categories;

            //console.log("seriesData="+JSON.stringify(seriesData));
            component.rootnodes(rootNodes.join());
            component.selecteddimension(component.dataRes.dimensions[component.selectedDimension].key);
            if (!component.includerootnodes() && (component.showtotals() && seriesData[0].data.length == 1 || !component.showtotals() && seriesData[0].data.length === 0)) {
                component.firePropertiesChangedAndEvent([
                    'rootnodes',
                    'selecteddimension'
                ], 'onHRootExpand');
            }

            return 0; //defaultValue.drilldown.series = ddsortedarray;
        };
    }

    if (validateImplementMethod(args, 'createSeriesData')) {
        /**
         * Map data from JSON Tuple to Highchart
         * @param {object} component - Reference to component
         * @param {string} type - type of the chart
         * @param {object} defaultValue - HighCharts values
         * @return {Array} seriesNames
         */
        that.createSeriesData = function(component, type, defaultValue) {
            //      alert('overriden');
            var self = this;
            var seriesColor = component.dataseriescolor().split(',');
            var measureDimensionIndex = self.getMeasureDimensionIndex(component);
            // measureDimensionIndex can be equal to -1 when measure is in external dimension.
            var measureInRow = (measureDimensionIndex !== -1 && component.dataRes.dimensions[measureDimensionIndex].axis == 'ROWS' ? true : false);
            component.categoriesIndex = [];
            //var columnCount = 0;
            //var curTuple;
            component.ztlSelectedRowJson = {};
            component.ztlSelectedColumnJson = {};
            //console.log("Series Data");
            //      alert('overridden');

            var defaultColor = [
                '#B586DA',
                '#FF6657',
                '#F3A730',
                '#946B66',
                '#B0AD53',
                '#7AC1DE',
                '#F1C337',
                '#566C6C',
                '#A1CC54',
                '#543F1E'
            ];
            seriesColor = [];
            if (component.dataseriescolor() === '' || component.dataseriescolor() === undefined) {
                seriesColor = [];
            } else {
                seriesColor = seriesColor.concat(component.dataseriescolor().split(','));
            }

            var noOfColumns = self.getNumberOfRowsColumns(component, "COLUMNS");
            /*var noOfColumns = component.dataRes['axis_columns'][0].indexOf(-1);
            if (noOfColumns == -1) {
              noOfColumns = component.dataRes['axis_columns'].length;
            }*/

            var selectedDimension = [];
            var measureColorPair = {};
            if (!(component.measurecolorpair() === undefined || component.measurecolorpair() === '')) {
                measureColorPair = JSON.parse(component.measurecolorpair());
            }
            //console.log("measureColorPair="+JSON.stringify(measureColorPair));
            if (!component.datautility() && type == 'bardd') {
                var seriesDataComplete = [];
                var categories = [];
                for (var i = noOfColumns; i < component.dataRes.dimensions.length; i++) {
                    selectedDimension.push(i);
                }

                if (selectedDimension.length < 2) {
                    return 1;
                }

                for (var col = 0; col < component.dataRes.columnCount; col++) {
                    if (self.vbiCharts.eliminateResult(component.dataRes, component.dataRes.tuples[col].slice(0, noOfColumns), undefined, component.showtotaldatabase())) {
                        var data = this.createDrilldownMultilevelData(component, selectedDimension, col, type);
                        seriesDataComplete.push(data[0][0]);
                    }
                }

                var SeriesAndDrilldownData = [seriesDataComplete /*,drilldownDataComplete*/ ];
                if (seriesDataComplete.length) {
                    $.each(seriesDataComplete[0].data, function(index, value) {
                        categories.push(value.name);
                    });
					/* jshint expr: true */ 
					categories.indexOf('Total') == -1 && component.showtotals() ? categories[component.totalnodeplacement() == 'before' ? 'unshift' : 'push']('Total') : ''; 
                    $.each(SeriesAndDrilldownData[0], function(index, value) {
                        value.color = measureColorPair[value.seriesKeyName] ? measureColorPair[value.seriesKeyName].color : self.getSeriesColors(component, seriesColor, index, defaultColor);
                    });

                }

                defaultValue.xAxis.categories = categories;
                defaultValue.series = SeriesAndDrilldownData[0];
                return 0;
            } else if (component.datautility()) {
                var bardatacustomizationJson = self.getBarDataCustomization(component);
                var selectedColumndimIndexes = [],
                    selectedRowdimIndexes = [];
                
                //var measureDimensionIndex=self.getMeasureDimensionIndex(component);
                if (measureInRow) {
                    selectedRowdimIndexes.push(measureDimensionIndex);
                } else if (measureDimensionIndex !== -1) {
                    selectedColumndimIndexes.push(measureDimensionIndex);
                }

                for (var i = 0; i < bardatacustomizationJson['ROWS'].length; i++) {
                    selectedRowdimIndexes.push(bardatacustomizationJson['ROWS'][i].index);
                }

                $.each(bardatacustomizationJson['COLUMNS'], function(index, value) {
                    selectedColumndimIndexes.push(_.parseInt(value.index));
                });

                if (selectedRowdimIndexes.length < 2) {
                    return 1;
                }

                var tuplesInARow = component.dataRes.tuples.slice(0, component.dataRes.columnCount);
                // Getting tuples for all Columns in first row of crosstab.
                var colRowIndexJson = {};
                var colNamesJson = {};
                var seriesData = [];
                var scalingValues = [],
                    unitOfMeasure = [];
				var hasResultIntupleCol = {};
				var showtotaldatabase = component.showtotaldatabase();
				var datautility = component.datautility();	

                if (measureDimensionIndex !== -1) {
                    for (var i = 0; i < component.dataRes.dimensions[measureDimensionIndex].members.length; i++) {
                        scalingValues.push(component.scalingfactorenable() ? component.dataRes.dimensions[measureDimensionIndex].members[i].scalingFactor || '' : '');
                        unitOfMeasure.push(component.unitcurrency() ? component.dataRes.dimensions[measureDimensionIndex].members[i].unitOfMeasure || '' : '');
                    }
                }


                for (var i = 0; i < tuplesInARow.length; i++) {
                    var tempTuple = tuplesInARow[i].slice(0, noOfColumns);
                    if (self.vbiCharts.eliminateResult(component.dataRes, tempTuple, undefined, component.showtotaldatabase()) === true) {
                        var colKey = [];
                        var colName = [];
                        var columnKey = [];
                        var tempZtlColJson = {};
                        for (var j = 0; j < selectedColumndimIndexes.length; j++) {
                            var tempCol = selectedColumndimIndexes[j];
                            colName.push(component.dataRes.dimensions[tempCol].members[tempTuple[tempCol]].text);
                            columnKey.push(component.dataRes.dimensions[tempCol].members[tempTuple[tempCol]].key.replace(/-/g, '_'));
                            colKey.push(tempCol + ':' + tempTuple[tempCol]);
                            tempZtlColJson[component.dataRes.dimensions[tempCol].key] = {
                                'internalKey': component.dataRes.dimensions[tempCol].members[tempTuple[tempCol]].key,
                                'text': component.dataRes.dimensions[tempCol].members[tempTuple[tempCol]]["picker_text"]
                            };
                        }

                        if (colKey.length === 0) {
                            colKey.push('none:none');
                            colName.push('Series-1');
                            columnKey.push('Series-1');
                        }

                        var newKey = colKey.join('-');
                        component.ztlSelectedColumnJson[columnKey.join('-')] = $.extend(true, {}, tempZtlColJson);
                        var scalingText = typeof tempTuple[measureDimensionIndex] == 'undefined' || scalingValues[tempTuple[measureDimensionIndex]] === '' ? '' : '(* ' + Math.pow(10, scalingValues[tempTuple[measureDimensionIndex]]) + ')';
                        var uintOfMeasureText = typeof tempTuple[measureDimensionIndex] == 'undefined' ? '' : unitOfMeasure[tempTuple[measureDimensionIndex]];
                        colName = _.filter(colName, function(x) {              
                          return (x != '' && x!= null && x!=" ");
                        });
                        var seriesName = colName.join('-');
                        if (!measureInRow && measureDimensionIndex !== -1) {
                            seriesName = self.formatSeriesName(component, seriesName, measureDimensionIndex + ':' + tempTuple[measureDimensionIndex], {
                                'showunit': 'unitcurrency'
                            });
                        }
                        //seriesName = seriesName + ' ' + uintOfMeasureText;
                        var sereisColumnKey = columnKey.join('-');
                        if (typeof colRowIndexJson[newKey] == 'undefined') {
                            colRowIndexJson[newKey] = [];
                            var colorvalue = measureColorPair[sereisColumnKey] ? measureColorPair[sereisColumnKey].color : self.getSeriesColors(component, seriesColor, seriesData.length, defaultColor);
                            colName = _.filter(colName, function(x) {              
                              return (x != '' && x!= null && x!=" ");
                            });
                            seriesData.push({
                                name: seriesName,
                                key: newKey,
                                color: colorvalue,
                                id: 'Series-' + newKey,
                                columnIndexKey: newKey,
                                columnKey: columnKey.join('-'),
                                seriesname: colName.join('-'),
                                seriesKeyName: sereisColumnKey,
                                unitOfMeasure: uintOfMeasureText,
                                scalingFactor: scalingText,
                                data: []
                            });
                        }

                        colRowIndexJson[newKey].push(i);
						/*#VBX141-4729*/
						if(datautility && showtotaldatabase){
							  hasResultIntupleCol[i] = [];
							  for(var j=0, jLen = tempTuple.length; j<jLen; j++){
								  if(component.dataRes.dimensions[j].members[tempTuple[j]].type == "RESULT"){					  
									  hasResultIntupleCol[i].push(j);
								  }
							  }
						 }
                        colNamesJson[newKey] = {
                            'text': seriesName,
                            'key': sereisColumnKey,
                            'unitOfMeasure': uintOfMeasureText,
                            'scalingFactor': scalingText
                        };
                    }
                }
                //console.log("col_row_index_json="+JSON.stringify(col_row_index_json));
                //console.log("colNamesJson="+JSON.stringify(colNamesJson));
                seriesColor.splice(seriesData.length, seriesColor.length);
                component.dataseriescolor(seriesColor.join());
                var tupleToSkip = 0;
                var tuplesInColumn = [];
                var rowNames = {};
                var rowDataJson = {};
				var rowDataJsonWithInfo = {};
				var rowDataJsonWithInfoDrilldown = {};
                var drilldownSeries = {};
                var positionJson = {};
				
				
                for (var i = 0; i < component.dataRes.rowCount; i++) {
                    //rowDataArr.push([]);
                    tupleToSkip = component.dataRes.columnCount * i;
                    tuplesInColumn.push(component.dataRes.tuples[tupleToSkip]);
                    var tempTuple = tuplesInColumn[i];
					/*#VBX141-4729*/
					var hasResultIntuple = [];
					if(datautility && showtotaldatabase){
						for(var j=noOfColumns, tLen = tempTuple.length; j< tLen; j++){
							if(component.dataRes.dimensions[j].members[tempTuple[j]].type == "RESULT"){
								hasResultIntuple.push(tempTuple[j]);
							}
						}
					}
                    if (self.vbiCharts.eliminateResult(component.dataRes, tempTuple, 'ROWS', component.showtotaldatabase()) === true) {
                        var rowKey = 0;
                        var rowHeader = 'Dimensions';
                        var rowKeyArr = [];
                        var rowKeyStr, rowKeyStrArr = [];
                        for (var j = 0; j < selectedRowdimIndexes.length; j++) {
                            if (j === 0) {
                                rowKey = selectedRowdimIndexes[0] + ':' + tempTuple[selectedRowdimIndexes[0]];
                                rowKeyStr = component.dataRes.dimensions[selectedRowdimIndexes[0]].key.replace(/-/g, '_') + ':' + component.dataRes.dimensions[selectedRowdimIndexes[0]].members[tempTuple[selectedRowdimIndexes[0]]].key.replace(/-/g, '_');
                                rowHeader = component.dataRes.dimensions[selectedRowdimIndexes[0]].members[tempTuple[selectedRowdimIndexes[0]]].text;

                            } else {
                                rowKey += '-' + selectedRowdimIndexes[j] + ':' + tempTuple[selectedRowdimIndexes[j]];
                                // jscs:disable
                                // Code here will be ignored by JSCS.
                                rowKeyStr += '-' + component.dataRes.dimensions[selectedRowdimIndexes[j]].key.replace(/-/g, '_') + ':' + component.dataRes.dimensions[selectedRowdimIndexes[j]].members[tempTuple[selectedRowdimIndexes[j]]].key.replace(/-/g, '_'); // jscs:enable
                                rowHeader = component.dataRes.dimensions[selectedRowdimIndexes[j]].members[tempTuple[selectedRowdimIndexes[j]]].text; // jscs:enable
                                //rowHeader += '\\' + component.dataRes.dimensions[selectedRowdimIndexes[j]].members[tempTuple[selectedRowdimIndexes[j]]].text;  // jscs:enable
                            }

                            rowKeyArr.push(rowKey);
                            rowKeyStrArr.push(rowKeyStr);
                            if (typeof rowNames[rowKey] == 'undefined') {
                                //rowDataJson[rowKey]=[];
                                rowNames[rowKey] = rowHeader;
                                if (typeof component.ztlSelectedRowJson[rowKeyStr] == 'undefined') {
                                    var dimKey = component.dataRes.dimensions[selectedRowdimIndexes[j]].key;
                                    component.ztlSelectedRowJson[rowKeyStr] = {};
                                    component.ztlSelectedRowJson[rowKeyStr][dimKey] = {
                                        internalKey: component.dataRes.dimensions[selectedRowdimIndexes[j]].members[tempTuple[selectedRowdimIndexes[j]]].key,
                                        text: component.dataRes.dimensions[selectedRowdimIndexes[j]].members[tempTuple[selectedRowdimIndexes[j]]]["picker_text"]
                                    };
                                }
                            }
                        }

                        $.each(colRowIndexJson, function(key, value) {
                            $.each(rowKeyArr, function(rowInd, ele) {
                                var seriesKeyName = colNamesJson[key].key;
                                var eleKey = rowKeyStrArr[rowInd];
                                var keygen = eleKey + '-' + seriesKeyName;
                                if (rowDataJson[keygen] === undefined) {
                                    rowDataJson[keygen] = [];
									rowDataJsonWithInfo[keygen] = [];																	
                                }

                                if (rowInd != rowKeyArr.length - 1) {
                                    if (!drilldownSeries[keygen]) {
                                        drilldownSeries[keygen] = {
                                            id: ele,
                                            name: colNamesJson[key].text,
                                            columnIndexKey: key,
                                            seriesname: colNamesJson[key].text,
                                            seriesKeyName: seriesKeyName,
                                            unitOfMeasure: colNamesJson[key].unitOfMeasure,
                                            scalingFactor: colNamesJson[key].scalingFactor,
                                            columnKey: seriesKeyName,
                                            value: [],
                                            data: []
                                        };
                                        positionJson[keygen] = {}; // all the sereis details
                                    }
                                }
								try{
                                var yArr = [];
								var yArrTotal = [];
                                $.each(value, function(index, newval) {
                                    var curIndex = newval + tupleToSkip;
                                    yArr.push(component.dataRes.data[curIndex] || 0);
                                    rowDataJson[keygen].push(component.dataRes.data[curIndex] || 0);
									
									/*#VBX141-4729*/
									if(datautility && showtotaldatabase && (hasResultIntuple.length || hasResultIntupleCol[newval].length)){	
										var lastTempStore, lastTempstoreLen = 0;
										var totalLen = hasResultIntupleCol[newval].length + hasResultIntuple.length;
										if(rowDataJsonWithInfo[keygen].length){
											lastTempStore = rowDataJsonWithInfo[keygen][rowDataJsonWithInfo[keygen].length - 1];
											lastTempstoreLen = Number(lastTempStore[0]);
										}
										if(lastTempstoreLen < totalLen){
											rowDataJsonWithInfo[keygen].length = 0;						
										}					
										rowDataJsonWithInfo[keygen].push([totalLen , (component.dataRes.data[curIndex] === null ? 0 : component.dataRes.data[curIndex])]);
										
										// processing yArrTotal
										lastTempstoreLen = 0;
										if(yArrTotal.length){
											lastTempStore = yArrTotal[yArrTotal.length - 1];
											lastTempstoreLen = Number(lastTempStore[0]);
										}
										if(lastTempstoreLen < totalLen){
											yArrTotal.length = 0;						
										}
										yArrTotal.push([totalLen, (component.dataRes.data[curIndex] === null ? 0 : component.dataRes.data[curIndex])]);
									}
									
                                });

                                if (rowInd > 0) {
                                    //var parentKey = ele.slice(0, ele.lastIndexOf('-'));
                                    var parentKey = eleKey.slice(0, eleKey.lastIndexOf('-'));
                                    var parentKeygen = parentKey + '-' + seriesKeyName;
                                    if (positionJson[parentKeygen].hasOwnProperty(eleKey)) {
                                        var currentYArr = drilldownSeries[parentKeygen].data[positionJson[parentKeygen][eleKey]].y;
                                        currentYArr = currentYArr.concat(yArr);
                                        drilldownSeries[parentKeygen].data[positionJson[parentKeygen][eleKey]].y = currentYArr;
                                    } else {
                                        positionJson[parentKeygen][eleKey] = drilldownSeries[parentKeygen].data.length;
                                        drilldownSeries[parentKeygen].backButton = component.dataRes.dimensions[ele.slice(ele.lastIndexOf('-') + 1, ele.length).split(':')[0]]["picker_text"];
                                        drilldownSeries[parentKeygen].data.push({
                                            drilldown: rowInd < rowKeyArr.length - 1 ? true : false,
                                            name: rowNames[ele],
                                            Namekey: eleKey,
                                            indexInfo: ele,
                                            parentKey: parentKey,
                                            level: rowInd,
                                            nodeState: 'collapsed',
                                            columnKey: colNamesJson[key].key,
                                            columnIndexKey: key,
                                            y: yArr
                                        });
                                    }						

                                    drilldownSeries[parentKeygen].value = drilldownSeries[parentKeygen].value.concat(yArr);
									
									
									if(typeof rowDataJsonWithInfoDrilldown[parentKeygen] === "undefined"){
										rowDataJsonWithInfoDrilldown[parentKeygen] = { "value" : [], "data" : []};
									}
									
									var pos = positionJson[parentKeygen][eleKey];
									rowDataJsonWithInfoDrilldown[parentKeygen].data[pos] = [];
									/*#VBX141-4729*/
									if(datautility && showtotaldatabase && yArrTotal.length){	
										var lastTempStore, lastTempstoreLen = 0;
										var totalLen = Number(yArrTotal[0][0]);
										
										// do for value
										if(rowDataJsonWithInfoDrilldown[parentKeygen].value.length){
											lastTempStore = rowDataJsonWithInfoDrilldown[parentKeygen].value[rowDataJsonWithInfoDrilldown[parentKeygen].value.length - 1];
											lastTempstoreLen = Number(lastTempStore[0]);
										}
										if(lastTempstoreLen < totalLen){
											rowDataJsonWithInfoDrilldown[parentKeygen].value.length = 0;						
										}										
										rowDataJsonWithInfoDrilldown[parentKeygen].value = rowDataJsonWithInfoDrilldown[parentKeygen].value.concat(yArrTotal);
										
										// do for data
										lastTempstoreLen = 0;										
										if(rowDataJsonWithInfoDrilldown[parentKeygen].data[pos].length){
											lastTempStore = rowDataJsonWithInfoDrilldown[parentKeygen].data[pos][rowDataJsonWithInfoDrilldown[parentKeygen].data[pos].length - 1];
											lastTempstoreLen = Number(lastTempStore[0]);
										}
										if(lastTempstoreLen < totalLen){
											rowDataJsonWithInfoDrilldown[parentKeygen].data[pos].length = 0;						
										}										
										rowDataJsonWithInfoDrilldown[parentKeygen].data[pos] = rowDataJsonWithInfoDrilldown[parentKeygen].data[pos].concat(yArrTotal);
										
										
									}							
                                }
                            }catch(e){
                            	console.log(e); // jshint ignore:line
                            }
                            });
                        });
                    }
                }
				
				/*#VBX141-4729*/
				if(datautility && showtotaldatabase){
					  $.each(rowDataJsonWithInfo, function(k, v){
						  if(v.length){
							  rowDataJson[k].length = 0;
								for(var i=0, len=v.length; i<len; i++){
									rowDataJson[k].push(Number(v[i][1]));
								}
						  }
					  });
					  
					  $.each(rowDataJsonWithInfoDrilldown, function(k, v){
						  if(v.value.length){
							  drilldownSeries[k].value.length = 0;
							  for(var i=0, len=v.value.length; i<len; i++){
									drilldownSeries[k].value.push(Number(v.value[i][1]));
								}
						  }
						  
						  if(v.data.length){
							  $.each(v.data, function(index, data){
								  if(data.length){
									  drilldownSeries[k].data[index].y.length = 0;
									  for(var i=0, len=data.length; i<len; i++){
										drilldownSeries[k].data[index].y.push(Number(data[i][1]));
									  }
								  }
							  });
						  }
					  });
				 }
				 
				 

                $.each(drilldownSeries, function(key, json) {
                    json.value = component.dataUtilityAggregation(json.value);
                    $.each(json.data, function(i, v) {
                        v.y = component.dataUtilityAggregation(v.y);
                    });
                });

                component.drilldownjson = drilldownSeries;
                //var categoryDataRowAggr = [];
                var finalCategoryNames = [];
                var srkey = '';
                var indexInfo = '';
                var totalArr = [];
                for (var i = 0; i < component.dataRes.dimensions[selectedRowdimIndexes[0]].members.length; i++) {
                    if (component.showtotaldatabase() || component.dataRes.dimensions[selectedRowdimIndexes[0]].members[i].type != 'RESULT') {
                        srkey = component.dataRes.dimensions[selectedRowdimIndexes[0]].key.replace(/-/g, '_') + ':' + component.dataRes.dimensions[selectedRowdimIndexes[0]].members[i].key.replace(/-/g, '_');
                        indexInfo = selectedRowdimIndexes[0] + ':' + i;
                        for (var j = 0; j < seriesData.length; j++) {
                            var genKey = srkey + '-' + seriesData[j].columnKey;
                            if (totalArr[j] === undefined) {
                                totalArr[j] = rowDataJson[genKey];
                            } else {
                                totalArr[j] = totalArr[j].concat(rowDataJson[genKey]);
                            }

                            var yVal = component.dataUtilityAggregation(rowDataJson[genKey]);
                            rowDataJson[genKey] = yVal;
                            seriesData[j].backButton = component.dataRes.dimensions[selectedRowdimIndexes[0]]["picker_text"];
                            seriesData[j].data.push({
                                Namekey: srkey,
                                indexInfo: indexInfo,
                                drilldown: true,
                                columnKey: seriesData[j].columnKey,
                                columnIndexKey: seriesData[j].columnIndexKey,
                                nodeState: 'collapsed',
                                level: 0,
                                parentKey: 'none',
                                name: component.dataRes.dimensions[selectedRowdimIndexes[0]].members[i].text,
                                y: yVal
                            });
                        }

                        finalCategoryNames.push(component.dataRes.dimensions[selectedRowdimIndexes[0]].members[i].text);
                    }
                }

                if (component.showtotals()) {
                    for (var j = 0; j < seriesData.length; j++) {
                        var tempTotal = {};
                        tempTotal.drilldown = false;
                        tempTotal.Namekey = 'total';
                        tempTotal.indexInfo = '-1:-1'; // index not available in resultset
                        tempTotal.columnKey = seriesData[j].columnKey;
                        tempTotal.columnIndexKey = seriesData[j].columnIndexKey;
                        tempTotal.nodeState = 'collapsed';
                        tempTotal.level = 0;
                        tempTotal.parentKey = 'none';
                        tempTotal.name = 'Total';
                        tempTotal.y = component.dataUtilityAggregation(totalArr[j]);
                        seriesData[j].data[component.totalnodeplacement() == 'before' ? 'unshift' : 'push'](tempTotal);
                    }

                    finalCategoryNames[component.totalnodeplacement() == 'before' ? 'unshift' : 'push']('Total');
                }

                defaultValue.xAxis.categories = finalCategoryNames;
                defaultValue.series = seriesData;
                return 0;
            }
        };
    }

    that.createDynamicDrilldownData = function(component, nodeKey) {
        var self = this;
        var level = 0,
            nextLevel = 0;
        var dimension = component.selectedDimension;
        component.setDataUtilityFormattedResultSet();
        var selectedMember = 0;
        var parentIndex = 0;
        var isParentSet = false;
        var sortedarray = [];
        var noOfColumns = self.getNumberOfRowsColumns(component, 'COLUMNS');
        var seriesNames;
        var seriesKeyNames;
        var seriesKeyIndexes;
        var seriesData = [];
        var dispKeyText = component.dispkeytext();
        var dispKeyTextNameJson = {};
        var allowednooflevels = Number(component.allowednooflevels());
        var type = component.getChartType();
        if (isNaN(allowednooflevels) || allowednooflevels < 2) {
            allowednooflevels = 1000;
        }

        if (component.includerootnodes()) {
            allowednooflevels = allowednooflevels - 1;
        }

        /**
         * VBX141-7796 
         * Making series consistent with other bar family charts where Measure Dimension is always taken first while making series name
         */
        var selectedColumndimIndexes = [];
        var measureDimensionIndex = 0;
        for (var i = 0; i < noOfColumns; i++) {
            if (component.dataRes.dimensions[i].containsMeasures === true) {
                selectedColumndimIndexes.unshift(i);
                measureDimensionIndex = i;
            }
            else{
                selectedColumndimIndexes.push(i);
            }
        }
        //setTimeout(function(){
        //console.log("**Result = "+ JSON.stringify(component.dataRes.dimensions));
        //},500);
        function getHierarchyDrilldownData(k, seriesKeyNames, tempSeriesKeyIndexes) {
            var series = {};
            var total = 0;
            var isBottom = component.nodeAlignment === 'bottom' ? true : false;
            var len = component.dataRes.dimensions[dimension].members.length;
            var memberKey;
            sortedarray = [];
            selectedMember = 0;
            series.nameKey = nodeKey;
                       
            for (var i = isBottom ? len-1 : 0; (isBottom ? i >= 0 : i<len) ; isBottom ? i-- : i++) {
                memberKey = component.dataRes.dimensions[dimension].members[i].originalKey || component.dataRes.dimensions[dimension].members[i].key;
                if (memberKey == nodeKey && component.dataRes.dimensions[dimension].members[i].type != 'HIERARCHY_NODE') {
                    return null;
                } else if (memberKey == nodeKey && component.dataRes.dimensions[dimension].members[i].type == 'HIERARCHY_NODE') {
                    /*if(!dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key])
                            {
                                var nameObj = $.extend(true,{},component.dataRes.dimensions[dimension].members[i]);
                                var name = that.dispKeyText(nameObj,dispKeyText,false);
                                dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key] = name;
                            }
                            series.name=dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key];*/
                    series.backButton = component.dataRes.dimensions[dimension].members[i]["picker_text"];
                    if (type != 'bardd') {
                        series.name = component.dataRes.dimensions[dimension].members[i].text;
                    }
                    //else
                    //{
                    //}
                    series.columnKey = seriesKeyNames;
                    series.columnIndexKey = tempSeriesKeyIndexes;
                    parentIndex = i;
                    if (typeof component.dataRes.dimensions[dimension].members[i].level != 'undefined') {
                        level = component.dataRes.dimensions[dimension].members[i].level;
                    }

                    nextLevel = level + 1;
                    isParentSet = true;
                } else if (( isBottom ? i < parentIndex : i > parentIndex) && isParentSet && (typeof component.dataRes.dimensions[dimension].members[i].level == 'undefined' ? 0 : component.dataRes.dimensions[dimension].members[i].level) == nextLevel) {
                    sortedarray[selectedMember] = {};
                    if (!dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key]) {
                        var nameObj = $.extend(true, {}, component.dataRes.dimensions[dimension].members[i]);
                        var name = that.dispKeyText(nameObj, dispKeyText, false);
                        dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key] = name;
                    }

                    sortedarray[selectedMember].name = dispKeyTextNameJson[component.dataRes.dimensions[dimension].members[i].key];
                    //sortedarray[selectedMember].name = component.dataRes.dimensions[dimension].members[i].text;
                    sortedarray[selectedMember].Namekey = component.dataRes.dimensions[dimension].members[i].originalKey || component.dataRes.dimensions[dimension].members[i].key;
                    sortedarray[selectedMember].indexInfo = dimension + ':' + i;
                    sortedarray[selectedMember].columnKey = seriesKeyNames;
                    sortedarray[selectedMember].columnIndexKey = tempSeriesKeyIndexes;
                    if (typeof component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey] == 'undefined') {
                        var dimKey = component.dataRes.dimensions[dimension].key;
                        component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey] = {};
                        component.ztlSelectedRowJson[sortedarray[selectedMember].Namekey][dimKey] = {
                            internalKey: component.dataRes.dimensions[dimension].members[i].originalKey || component.dataRes.dimensions[dimension].members[i].key,
                            text: component.dataRes.dimensions[dimension].members[i]["picker_text"]
                        };
                    }

                    sortedarray[selectedMember].parentKey = nodeKey;
                    sortedarray[selectedMember].nodeState = 'collapsed';
                    sortedarray[selectedMember].level = nextLevel;
                    sortedarray[selectedMember].drilldown = component.dataRes.dimensions[dimension].members[i].type == 'HIERARCHY_NODE' && allowednooflevels > nextLevel ? true : false;
                    sortedarray[selectedMember].y = [];
                    //sortedarray[selectedMember].selectedColumnIndex=selectedColumnIndex;
                    for (var j = k; j < component.dataRes.data.length; j += component.dataRes.columnCount) {
                        if (i == component.dataRes.tuples[j][dimension] && self.vbiCharts.eliminateResult(component.dataRes, component.dataRes.tuples[j], undefined, component.showtotaldatabase()) === true) {
                            sortedarray[selectedMember].y.push(component.dataRes.data[j]);
                        }
                    }

                    sortedarray[selectedMember].y = component.dataUtilityAggregation(sortedarray[selectedMember].y);
                    total += sortedarray[selectedMember].y;
                    selectedMember++;
                } else if (( isBottom ? i < parentIndex : i > parentIndex) && isParentSet && level == (typeof component.dataRes.dimensions[dimension].members[i].level == 'undefined' ? 0 : component.dataRes.dimensions[dimension].members[i].level)) {
                    break;
                }
            }
            
            series.value = total;
            series.data = sortedarray;
            return series;
        }

        if (component.datautility() === false && type == 'bardd') {
            seriesData = [];
            for (var k = 0; k < component.dataRes.columnCount; k++) {
                var curTuple = component.dataRes.tuples[k];
                var tempSeriesNames = [];
                var tempSeriesKeys = [];
                var tempSeriesKeyIndexes = [];
                var unitOfMeasure = '';
                var scalingFactor = 0;
                var measureDimIndex = '';
                for (var i=0, j = 0; i < selectedColumndimIndexes.length; i++) {
                    j = selectedColumndimIndexes[i];
                    if (component.showtotaldatabase() || component.dataRes.dimensions[j].members[curTuple[j]].type != 'RESULT') {
                        if (j === measureDimensionIndex) {
                            measureDimIndex = j + ':' + curTuple[j];

                            if (component.unitcurrency()) {
                                unitOfMeasure = component.dataRes.dimensions[j].members[curTuple[j]].unitOfMeasure || '';
                            }

                            if (component.scalingfactorenable()) {
                                scalingFactor = component.dataRes.dimensions[j].members[curTuple[j]].scalingFactor || 0;
                            }
                        }
                        tempSeriesNames.push(component.dataRes.dimensions[j].members[curTuple[j]].text);
                        tempSeriesKeys.push(component.dataRes.dimensions[j].members[curTuple[j]].key);
                        tempSeriesKeyIndexes.push(j + ':' + curTuple[j]);
                    } else {
                        tempSeriesNames = [];
                        tempSeriesKeys = [];
                        tempSeriesKeyIndexes = [];
                        break;
                    }
                }

                if (selectedColumndimIndexes.length === 0 || tempSeriesNames.length !== 0) {
                    tempSeriesNames = _.filter(tempSeriesNames, function(x) {              
                      return (x != '' && x!= null && x!=" ");
                    });
                    seriesNames = tempSeriesNames.join('-');
                    seriesKeyNames = tempSeriesKeys.join('-');
                    seriesKeyIndexes = tempSeriesKeyIndexes.join('-');
                    if(measureDimIndex){
                        seriesNames = self.formatSeriesName(component, seriesNames, measureDimIndex, {
                            'showunit': 'unitcurrency'
                        });
                    }

                    if (seriesKeyNames === '') {
                        seriesKeyIndexes = 'none:none';
                        seriesKeyNames = 'Series-1';
                        seriesNames = 'Series-1';
                    }



                    /*if(scalingFactor)
                    {
                         seriesNames+= " (*"+Math.pow(10, scalingFactor) + ")";
                    }
                    if(unitOfMeasure)
                    {
                          seriesNames+=" (in"+unitOfMeasure+")";
                    }*/
                    var returnedSeries = getHierarchyDrilldownData(k, seriesKeyNames, seriesKeyIndexes);
                    returnedSeries.seriesname = seriesNames;
                    returnedSeries.name = seriesNames;
                    returnedSeries.seriesKeyName = seriesKeyNames;
                    returnedSeries.unitOfMeasure = unitOfMeasure;
                    returnedSeries.scalingFactor = Math.pow(10, scalingFactor);
                    seriesData.push(returnedSeries);
                }
            }
        } else {
            var ColumnDimensionCount = component.dataRes['axis_columns'][0].indexOf(-1);
			var selectedColumnIndex = 0;
            if (type == 'funneldd' || type == 'piedd') {
                seriesData = [];
                selectedColumnIndex = 0;
                if (component.selectedColumns.length > 0) {
                    for (var i = 0; i < component.dataRes.tuples.length; i++) {
                        if (component.dataRes.tuples[i].slice(0, ColumnDimensionCount).toString() == component.columnDimensionIndex[component.selectedColumnsKey[0]].index.toString()) {
                            selectedColumnIndex = i;
                            break;
                        }
                    }
                }

                var returnedSeries = getHierarchyDrilldownData(selectedColumnIndex, component.selectedColumnsKey[0], component.selectedColumnsIndexKey[0]);
                returnedSeries.seriesname = component.selectedColumns[0];
                returnedSeries.seriesKeyName = component.selectedColumnsKey[0];
                returnedSeries.unitOfMeasure = component.unitOfMeasure[0];
                returnedSeries.scalingFactor = Math.pow(10, component.scalingFactor[0]);
                seriesData.push(returnedSeries);
                //console.log("seriesData="+JSON.stringify(seriesData));
                if (type == 'funneldd') {
                    for (var i = 0; i < seriesData[0].data.length; i++) {
                        for (var j = 0; j < seriesData[0].data.length - 1; j++) {
                            if (seriesData[0].data[i].y > seriesData[0].data[j].y) {
                                var temp = seriesData[0].data[i];
                                seriesData[0].data[i] = seriesData[0].data[j];
                                seriesData[0].data[j] = temp;
                            }
                        }
                    }
                }
            } else {
                seriesData = [];
                selectedColumnIndex = 0;
                $.each(component.selectedColumns, function(k, v) {
                    for (var i = 0; i < component.dataRes.tuples.length; i++) {
                        if (component.dataRes.tuples[i].slice(0, ColumnDimensionCount).toString() == component.columnDimensionIndex[v].index.toString()) {
                            selectedColumnIndex = i;
                            break;
                        }
                    }

                    var returnedSeries = getHierarchyDrilldownData(selectedColumnIndex, component.selectedColumnsKey[k], component.selectedColumnsIndexKey[k]);
                    returnedSeries.seriesname = component.selectedColumns[k];
                    returnedSeries.name = component.selectedColumns[k];
                    returnedSeries.seriesKeyName = component.selectedColumnsKey[k];
                    returnedSeries.scalingFactor = Math.pow(10, component.scalingFactor[k]);
                    returnedSeries.unitOfMeasure = component.unitOfMeasure[k];
                    seriesData.push(returnedSeries);
                });
            }
        }

        return seriesData;
    };

    if (validateImplementMethod(args, 'expandCollapseHandler')) {
        that.expandCollapseHandler = function(component) {
            var expandORCollapse = '';
            if (component.isHierarchy) {
                if (component.eventPoint.drilldown !== false) {
                    if (component.eventPoint.nodeState == 'collapsed') {
                        if (typeof component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey] == 'undefined') {
                            component.selectednode(component.nodeKey);
                            component.selecteddimension(component.dataRes.dimensions[component.selectedDimension].key);
                            component.firePropertiesChangedAndEvent([
                                'selectednode',
                                'selecteddimension'
                            ], 'onHExpand');
                            component.expandflag = 1;
                        } else {
                            that.expandNode(component);
                        }

                        expandORCollapse = 'expand';
                    } else {
                        that.collapseNode(component);
                        expandORCollapse = 'collapse';
                    }
                }
            } else {
                if (component.eventPoint.drilldown !== false) {
                    if (component.eventPoint.nodeState == 'collapsed') {
                        that.expandNode(component);
                        expandORCollapse = 'expand';
                    } else {
                        that.collapseNode(component);
                        expandORCollapse = 'collapse';
                    }
                }
            }



            var eventToTrigger = expandORCollapse == 'expand' ? 'chartonexpand' : 'chartoncollapse';

            if (expandORCollapse == 'expand') {
                that.trackActionFunc(component, "expand");
            } else {
                that.trackActionFunc(component, "collapse");
            }


            if (component.customAction === null && component[eventToTrigger]() !== '') {
                if (component.nodeKey != 'total') {
                    try {
                        var ztlPointJson = {};
                        //var prevSelectedVal = component.selectedvalue;
                        ztlPointJson = $.extend(true, {}, component.ztlSelectedColumnJson[component.seriesColumnkey]);
                        $.extend(true, ztlPointJson, component.ztlSelectedRowJson[component.nodeKey]);
                        component.selectedvalue(JSON.stringify(ztlPointJson));
                    } catch (e) {}
                }
                component.columncolorChanged++;
                component.firePropertiesChangedAndEvent([
                    'selectedvalue',
                    'selectedlevel'
                ], eventToTrigger);
            }
            /*if (expandORCollapse == 'expand') {
              component.fireEvent('chartonexpand');
            } else if (expandORCollapse == 'collapse') {
              component.fireEvent('chartoncollapse');
            }*/
        };
    }

    if (validateImplementMethod(args, 'hierarchyDrilldownFromJson')) {
        that.hierarchyDrilldownFromJson = function(component) {
			var self = this;
            var groupedCategories = [];
            var mainseriesWithParents = {};
            var currentSeriesIndex = component.eventPoint.columnIndexKey;
            var allSeriesData = [];
            var chartType = component.getChartType();
			var allSeries = [];
			var seriesWithParents = {};

            that.trackActionFunc(component, "drilldown");

            if (chartType == 'bardd') {
                $.extend(true, allSeriesData, component.eventPoint.series.chart.series);
                $.each(component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey].data, function(i, v) {
                    groupedCategories.push(v.name);
                });
            }

            if (component.parentondrill()) {
                $.extend(true, mainseriesWithParents, component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey]);
                mainseriesWithParents.data[component.parentnodeplacement() == 'before' ? 'unshift' : 'push']({
                    name: component.eventPoint.name,
                    columnIndexKey: component.eventPoint.columnIndexKey,
                    columnKey: component.seriesColumnkey,
                    Namekey: component.eventPoint.Namekey,
                    parentKey: component.eventPoint.parentKey,
                    nodeState: component.eventPoint.nodeState,
                    level: component.eventPoint.level,
                    drilldown: false,
                    y: component.eventPoint.y
                });
                if (chartType == 'bardd') {
                    groupedCategories[component.parentnodeplacement() == 'before' ? 'unshift' : 'push'](component.eventPoint.name);
                }
            } else {
                $.extend(true, mainseriesWithParents, component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey]);
            }

            if (chartType == 'bardd') {
                allSeries = [];
                component.eventPoint.series.xAxis.setCategories(groupedCategories);
				self.updateCategoryWidthHeight(component, groupedCategories);
                if (component.showallmeasures()) {
                    //console.log(JSON.stringify(allSeriesData));
                    for (var k = 0; k < allSeriesData.length; k++) {
                        seriesWithParents = {};
                        if (allSeriesData[k].columnIndexKey != currentSeriesIndex) {
                            var seriesKeyName = allSeriesData[k].options.seriesKeyName;
                            if (component.parentondrill()) {
                                $.extend(true, seriesWithParents, component.drilldownjson[component.nodeKey + '-' + seriesKeyName]);
                                seriesWithParents.color = allSeriesData[k].color;
                                for (var i = 0; i < allSeriesData[k].data.length; i++) {
                                    //console.log("in for = "+i+"  allSeriesData[k].data[i].Namekey="+allSeriesData[k].data[i].Namekey+"  component.eventPoint.Namekey="+component.eventPoint.Namekey);
                                    if (allSeriesData[k].data[i].Namekey == component.eventPoint.Namekey) {
                                        //console.log("allSeriesData[k].data[i].Namekey="+allSeriesData[k].data[i].Namekey + " seriesKeyName="+seriesKeyName);
                                        seriesWithParents.data[component.parentnodeplacement() == 'before' ? 'unshift' : 'push']({
                                            name: component.eventPoint.name,
                                            columnKey: seriesKeyName,
                                            columnIndexKey: allSeriesData[k].data[i].columnIndexKey,
                                            Namekey: component.eventPoint.Namekey,
                                            parentKey: allSeriesData[k].data[i].parentKey,
                                            nodeState: allSeriesData[k].data[i].nodeState,
                                            level: allSeriesData[k].data[i].level,
                                            drilldown: false,
                                            y: component.drilldownjson[component.nodeKey + '-' + seriesKeyName].value
                                        });
                                        break;
                                    }
                                }

                                allSeries.push(seriesWithParents);
                            } else {
                                $.extend(true, seriesWithParents, component.drilldownjson[component.nodeKey + '-' + seriesKeyName]);
                                seriesWithParents.color = allSeriesData[k].color;
                                allSeries.push(seriesWithParents);
                            }
                        } else if (allSeriesData[k].columnIndexKey == currentSeriesIndex) {
                            allSeries.push(mainseriesWithParents);
                        }
                    }

                    component.chartRef.addMultipleSeriesAsDrilldown(component.eventPoint, allSeries);
                } else {
                    component.chartRef.addSeriesAsDrilldown(component.eventPoint, mainseriesWithParents);
                }
            } else {
                component.chartRef.addSeriesAsDrilldown(component.eventPoint, mainseriesWithParents);
            }

            allSeries = null;
            groupedCategories = null;
            seriesWithParents = null;
            component.clickEvent = ""; // clearing the event
            if (component.customAction !== null) {
                that.rebuildActionsRecursive(component);
            }

        };
    }

    if (validateImplementMethod(args, 'getAllChildren')) {
        that.getAllChildren = function(component, node, seriesColumnkey, gpc, nodeState) {
            if (typeof component.drilldownjson[node + '-' + seriesColumnkey] != 'undefined') {
                $.each(component.drilldownjson[node + '-' + seriesColumnkey].data, function(i, v) {
                    if (typeof component.drilldownjson[v.Namekey + '-' + seriesColumnkey] != 'undefined' && (v.Namekey != component.nodeKey && v.nodeState == 'expanded' || v.Namekey == component.nodeKey && nodeState == 'expanded')) {
                        gpc.categories.push({
                            name: v.name,
                            level: v.level,
                            categories: [v.name]
                        });
                    } else {
                        gpc.categories.push(v.name);
                    }

                    var parentgpc = gpc;
                    gpc = gpc.categories[gpc.categories.length - 1];
                    if (typeof gpc == 'object') {
                        that.getAllChildren(component, v.Namekey, seriesColumnkey, gpc, nodeState);
                        if (component.parentnodeplacement() == 'after') {
                            var categories = gpc.categories;
                            var parentNode = categories.shift();
                            categories.push(parentNode);
                        }
                    }

                    gpc = parentgpc;
                });
            }
        };
    }

    if (validateImplementMethod(args, 'createFlatCategory')) {
        that.createFlatCategory = function(flat, gpc) {
            if (flat.categories) {
                $.each(flat.categories, function(i, v) {
                    if (typeof v == 'object') {
                        that.createFlatCategory(v, gpc);
                    } else {
                        gpc.push(v);
                    }
                });
            } else {
                gpc.push(flat);
            }
        };
    }

    if (validateImplementMethod(args, 'expandNode')) {
        that.expandNode = function(component) {
			var self = this;
            var currentSeries = component.eventPoint.series.data;
            var allSeriesData = component.eventPoint.series.chart.series;
            var currentSeriesIndex = component.eventPoint.columnIndexKey;
            var chartType = component.getChartType();
            //var currentIndex = eventPoint.index;
            var newSeries = [];
            var groupedCategories = [];
            var xAxisMembers = [];
            var posParentInSeries = -1; // Position of a node which is parent to current node
            $.each(currentSeries, function(index, value) {
                xAxisMembers.push(value.Namekey);
            });

            function getNodeChildren(nodeKey, columnKey){
                var parentNode;
                if (component.parentnodeplacement() == 'after') {
                    parentNode = newSeries.pop();
                }
                $.each(component.drilldownjson[nodeKey + '-' + columnKey].data, function(i, v) {
                    newSeries.push(v);
                    if(v.nodeState === 'expanded'){
                        getNodeChildren(v.Namekey, columnKey);
                    }
                });
                if (parentNode) {
                    newSeries.push(parentNode);
                }               
            }

            //setTimeout(function() {
            $.each(currentSeries, function(index, value) {
                newSeries.push({
                    Namekey: value.Namekey,
                    columnKey: value.columnKey,
                    columnIndexKey: value.columnIndexKey,
                    drilldown: value.drilldown,
                    parentKey: value.parentKey,
                    name: value.name,
                    y: value.y,
                    nodeState: value.nodeState,
                    level: value.level
                });
                posParentInSeries = -1;

                /*
                  1. check is parent is none :- Expand/Collapse starts from level 0
                  2. Situation occurs when drilldown( show parent node on drilldown is enabled) --> expand :-
                        Expand/Collapse does not start from level 0 and parent node of the current node is not present
                  3. Situation occurs when drilldown( show parent node on drilldown is enabled) --> expand :-
                        Expand/Collapse does not start from level 0 and parent node of the current node is present.
                        Hence the parent node is checked wether it is collapsed.

                  #VBX141-4001 :- Solution is condition 3.
                */
                if (value.parentKey == 'none' || (posParentInSeries = xAxisMembers.indexOf(value.parentKey)) == -1 || currentSeries[posParentInSeries].nodeState == 'collapsed') {
                    if (value.nodeState == 'expanded' || component.nodeKey == value.Namekey) {
                        groupedCategories.push({
                            name: value.name,
                            level: value.level,
                            categories: [value.name]
                        });
                        that.getAllChildren(component, value.Namekey, component.seriesColumnkey, groupedCategories[groupedCategories.length - 1], 'expanded');
                        if (component.parentnodeplacement() == 'after') {
                            var categories = groupedCategories[groupedCategories.length - 1].categories;
                            var parentNode = categories.shift();
                            categories.push(parentNode);
                        }
                    } else {
                        groupedCategories.push(value.name);
                    }
                }

                if (component.nodeKey == value.Namekey) {
                    newSeries[newSeries.length - 1].nodeState = 'expanded';
                    if (value.parentKey != 'none') {
                        $.each(component.drilldownjson[value.parentKey + '-' + component.seriesColumnkey].data, function(i, v) {
                            if (v.Namekey == component.nodeKey && component.seriesColumnkey == v.columnKey) {
                                v.nodeState = 'expanded';
                            }
                        });
                    }
                    /**
                     * #VBX141-8191
                     */
                    getNodeChildren(component.nodeKey, component.seriesColumnkey);
                }
            });
            // setting data in other series
            if (chartType == 'bardd') {
                if (!component.activatehierarchylabeling()) {
                    var flatCategories = $.extend(true, {}, groupedCategories);
                    groupedCategories = [];
                    $.each(flatCategories, function(i, v) {
                        that.createFlatCategory(v, groupedCategories);
                    });
                }

                for (var k = 0; k < allSeriesData.length; k++) {
                    if (allSeriesData[k].userOptions.columnIndexKey != currentSeriesIndex) {
                        var seriesObjData = allSeriesData[k].data;
                        var seriesKeyName = allSeriesData[k].userOptions.seriesKeyName;
                        var tempSeries = [];
                        $.each(seriesObjData, function(index, value) {
                            tempSeries.push({
                                Namekey: value.Namekey,
                                columnKey: value.columnKey,
                                columnIndexKey: value.columnIndexKey,
                                drilldown: value.drilldown,
                                parentKey: value.parentKey,
                                name: value.name,
                                y: value.y,
                                nodeState: value.nodeState,
                                level: value.level
                            });
                            if (component.nodeKey == value.Namekey) {
                                if (tempSeries[tempSeries.length - 1].nodeState == 'collapsed') {
                                    tempSeries[tempSeries.length - 1].nodeState = 'expanded';
                                    if (value.parentKey != 'none') {
                                        $.each(component.drilldownjson[value.parentKey + '-' + seriesKeyName].data, function(i, v) {
                                            if (v.Namekey == component.nodeKey && seriesKeyName == v.columnKey) {
                                                v.nodeState = 'expanded';
                                            }
                                        });
                                    }

                                    var localMemberjson = [];
                                    $.extend(true, localMemberjson, component.drilldownjson[component.nodeKey + '-' + seriesKeyName].data);
                                    if (component.parentnodeplacement() == 'after') {
                                        var parentNode = tempSeries.pop();
                                        $.each(localMemberjson, function(i, v) {
                                            if (!component.showallmeasures()) {
                                                v.y = 0;
                                            }

                                            tempSeries.push(v);
                                        });

                                        tempSeries.push(parentNode);
                                    } else {
                                        $.each(localMemberjson, function(i, v) {
                                            if (!component.showallmeasures()) {
                                                v.y = 0;
                                            }

                                            tempSeries.push(v);
                                        });
                                    }
                                }
                            }
                        });

                        allSeriesData[k].setData(tempSeries, false);
                    }
                }

                component.eventPoint.series.xAxis.setCategories(groupedCategories);
				self.updateCategoryWidthHeight(component, groupedCategories);
            }

            component.eventPoint.series.setData(newSeries);
            //}, 200);
            component.clickEvent = ""; // clearing the event

            if (component.customAction !== null) {
                that.rebuildActionsRecursive(component);
            }
        };
    }

    if (validateImplementMethod(args, 'collapseNode')) {
        that.collapseNode = function(component) {
			var self = this;
            var currentSeries = component.eventPoint.series.data;
            //var currentIndex = component.eventPoint.index;
            //var seriesObj = component.eventPoint.series;
            var groupedCategories = [];
            var seriesKeyName = component.seriesColumnkey;
            var allSeriesData = component.eventPoint.series.chart.series;
            var currentSeriesIndex = component.eventPoint.columnIndexKey;
            var chartType = component.getChartType();

            function collapseAllChildren(nodesJsonArr, seriesKeyName) {
                $.each(nodesJsonArr, function(i, v) {
                    v.nodeState = 'collapsed';
                    if (typeof component.drilldownjson[v.Namekey + '-' + seriesKeyName] != 'undefined') {
                        collapseAllChildren(component.drilldownjson[v.Namekey + '-' + seriesKeyName].data, seriesKeyName);
                    }
                });
            }

            function collpaseSeriesBySeries(seriesObj) {
                var removeFlag = 0;
                var newSeries = [];
                var posParentInSeries = -1;
                currentSeries = seriesObj.data;
                if (component.parentnodeplacement() == 'after') {
                    currentSeries.reverse();
                }

                var xAxisMembers = [];
                $.each(currentSeries, function(index, value) {
                    xAxisMembers.push(value.Namekey);
                });

                seriesKeyName = seriesObj.userOptions.seriesKeyName;
                $.each(currentSeries, function(index, value) {
                    if (removeFlag === 0 || removeFlag == 1 && component.nodeLevel >= value.level) {
                        removeFlag = 0;
                        newSeries.push({
                            Namekey: value.Namekey,
                            columnKey: value.columnKey,
                            columnIndexKey: value.columnIndexKey,
                            drilldown: value.drilldown,
                            parentKey: value.parentKey,
                            name: value.name,
                            y: value.y,
                            nodeState: value.nodeState,
                            level: value.level
                        });
                        posParentInSeries = -1;
                        if ((value.parentKey == 'none' || (posParentInSeries = xAxisMembers.indexOf(value.parentKey)) == -1 || currentSeries[posParentInSeries].nodeState == 'collapsed') && seriesObj.userOptions.columnIndexKey == currentSeriesIndex) {
                            if (value.nodeState == 'expanded' && value.Namekey != component.nodeKey) {
                                groupedCategories.push({
                                    name: value.name,
                                    level: value.level,
                                    categories: [value.name]
                                });
                                that.getAllChildren(component, value.Namekey, seriesKeyName, groupedCategories[groupedCategories.length - 1], 'collapsed');
                                if (component.parentnodeplacement() == 'after') {
                                    var categories = groupedCategories[groupedCategories.length - 1].categories;
                                    var parentNode = categories.shift();
                                    categories.push(parentNode);
                                }
                            } else {
                                groupedCategories.push(value.name);
                            }
                        }
                    }

                    if (component.nodeKey == value.Namekey && seriesKeyName == value.columnKey) {
                        if (value.parentKey != 'none') {
                            $.each(component.drilldownjson[value.parentKey + '-' + seriesKeyName].data, function(i, v) {
                                if (v.Namekey == component.nodeKey && seriesKeyName == v.columnKey) {
                                    v.nodeState = 'collapsed';
                                    if (typeof component.drilldownjson[component.nodeKey + '-' + seriesKeyName] != 'undefined') {
                                        collapseAllChildren(component.drilldownjson[component.nodeKey + '-' + seriesKeyName].data, seriesKeyName);
                                    }
                                }
                            });
                        } else {
                            if (typeof component.drilldownjson[component.nodeKey + '-' + seriesKeyName] != 'undefined') {
                                collapseAllChildren(component.drilldownjson[component.nodeKey + '-' + seriesKeyName].data, seriesKeyName);
                            }
                        }

                        newSeries[newSeries.length - 1].nodeState = 'collapsed';
                        removeFlag = 1;
                    }
                });

                if (component.parentnodeplacement() == 'after') {
                    newSeries.reverse();
                    groupedCategories.reverse();
                }

                if (seriesObj.userOptions.columnIndexKey != currentSeriesIndex) {
                    seriesObj.setData(newSeries, false);
                } else {
                    if (chartType == 'bardd') {
                        if (!component.activatehierarchylabeling()) {
                            var flatCategories = $.extend(true, {}, groupedCategories);
                            groupedCategories = [];
                            $.each(flatCategories, function(i, v) {
                                that.createFlatCategory(v, groupedCategories);
                            });
                        }

                        seriesObj.xAxis.setCategories(groupedCategories);
						self.updateCategoryWidthHeight(component, groupedCategories);
                    }

                    seriesObj.setData(newSeries);
                }
            }

            if (chartType == 'bardd') {
                for (var k = 0; k < allSeriesData.length; k++) {
                    if (allSeriesData[k].userOptions.columnIndexKey != currentSeriesIndex) {
                        collpaseSeriesBySeries(allSeriesData[k]);
                    }
                }
            }

            collpaseSeriesBySeries(component.eventPoint.series);
            component.clickEvent = ""; // clearing the event
        };
    }

    /*that.decideEventType = function(event, component){
          var eventMap = {
                'ondblclickevent' : 'ondbltapevent',
                'onclickevent' : 'ontapevent'
          }
          if(component.configuremobileevent() && component._isIOS){
              return eventMap[event];
          }
          return event;
    };*/
    /* modify highchart default single click and implement double click*/
    that._pointClickEvent = function(event, component) {
        if(event.point){
        try{
            component.eventPoint = event.point;
            component.nodeLevel = event.point.level;
            component.selectedlevel(Number(component.nodeLevel));
            component.seriesColumnkey = event.point.columnKey;
            component.nodeKey = event.point.Namekey;
            component.parentKey = event.point.parentKey;
            component.clickEvent = 'ondblclickevent'; //that.decideEventType('ondblclickevent', component);
            var singleClick = function() {
                var clickEvent = component.clickEvent;
                if (component[clickEvent]() == 'none') {
                    return;
                } else if (component[clickEvent]() == 'expand_collapse') {
                    if (event.point.Namekey) {
                        component.eventPoint.select(!1, !1);
                        that.expandCollapseHandler(component);
                    }
                } else if (component[clickEvent]() == 'select') {
                    var chartType = component.getChartType();
                    //#VBX141-7966 : If shift key is not pressed during select then deselect all other points.
                    if (event.shiftKey) {
                        component.eventPoint.select(null, !0);
                    }
                    else{
                        component.eventPoint.select(null, !1);
                    }
                    
                    var selectedPoints = component.chartRef.getSelectedPoints();
                    if (component.nodeKey != 'total') {
                        try {
                            var ztlPointJson = {};
                            if (component.eventPoint.selected) {
                                that.trackActionFunc(component, "select");
                                ztlPointJson = $.extend(true, {}, component.ztlSelectedColumnJson[component.seriesColumnkey]);
                                $.extend(true, ztlPointJson, component.ztlSelectedRowJson[component.nodeKey]);
                                component.selectedvalue(JSON.stringify(ztlPointJson));
                            } else {

                                that.trackActionFunc(component, "deselect");
                                
                                /*
                                #VBX141-4144
                                ztlPointJson = $.extend(true, {}, component.ztlSelectedColumnJson[component.seriesColumnkey]);
                                $.extend(true, ztlPointJson, component.ztlSelectedRowJson[component.nodeKey]);
                                $.each(ztlPointJson, function(k) {
                                    ztlPointJson[k] = {
                                        'internalKey': '',
                                        'text': ''
                                    };
                                });
                                component.selectedvalue(JSON.stringify(ztlPointJson));*/
                                component.selectedvalue('');
                            }

                            var pointMemberJson = [];
                            $.each(selectedPoints, function(i, v) {
                                ztlPointJson = {};
                                ztlPointJson = $.extend(true, {}, component.ztlSelectedColumnJson[v.columnKey]);
                                $.extend(true, ztlPointJson, component.ztlSelectedRowJson[v.Namekey]);
                                pointMemberJson.push(JSON.stringify(ztlPointJson));
                            });

                            component.selectedvalues(JSON.stringify(pointMemberJson));
                            component.columncolorChanged++;
                        } catch (e) {}
                    }

                    component.firePropertiesChangedAndEvent([
                        'selectedvalue',
                        'selectedvalues',
                        'selectedlevel'
                    ], 'chartonpointclick');
                } else if (component[clickEvent]() == 'drilldown' && component.eventPoint.drilldown !== false) {
                    if (!event.seriesOptions && component.isHierarchy) {
                        if (typeof component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey] === 'undefined') {
                            component.selectednode(component.nodeKey);
                            component.selecteddimension(component.dataRes.dimensions[component.selectedDimension].key);
                            component.firePropertiesChangedAndEvent([
                                'selectednode',
                                'selecteddimension'
                            ], 'onHExpand');
                            component.drilldownflag = 1;
                        } else {
                            that.hierarchyDrilldownFromJson(component);
                        }
                    } else {
                        that.hierarchyDrilldownFromJson(component);
                    }
                    //condition for updating the drilldown level when datasource is changed and chart is in 0 the level
                    if (component.drilldownlevel() == 1 && Number(component.nodeLevel) === 0) {
                        component.drilldownlevel(0);
                    } else {
                        component.drilldownlevel(Number(component.nodeLevel) + 1);
                    }

                    component.drillupArr.push({
                        'column': component.seriesColumnkey,
                        'row': component.nodeKey
                    });

                    if (component.customAction === null) {
                        if (component.chartondrilldown() !== '') {
                            try {
                                var ztlPointJson = {};
                                ztlPointJson = $.extend(true, {}, component.ztlSelectedColumnJson[component.seriesColumnkey]);
                                $.extend(true, ztlPointJson, component.ztlSelectedRowJson[component.nodeKey]);
                                component.selectedvaluedrilldown(JSON.stringify(ztlPointJson));
                                component.columncolorChanged++;

                            } catch (e) {}

                            component.firePropertiesChangedAndEvent([
                                'selectedvaluedrilldown',
                                'selectedlevel',
                                'drilldownlevel'
                            ], 'chartondrilldown');
                        } else {
                            component.columncolorChanged++;
                            component.firePropertiesChanged(['selectedlevel', 'drilldownlevel']);
                        }
                       // console.log(component.selectedvaluedrilldown())
                    }

                }
            };
            var setClickevent = function() {
                component.clickEvent = 'onclickevent'; //that.decideEventType('onclickevent', component);
                if (event.ctrlKey || event.metaKey /*|| event.shiftKey */) { //#VBX141-7966 : Commenting shiftKey condition, as 'select' can be associated with any event.
                    component.clickEvent = 'onctrlclickevent';
                }

                component.clicks = 0;
                singleClick();
            };

            var setDblClickEvent = function() {
                component.clickEvent = 'ondblclickevent'; //that.decideEventType('ondblclickevent', component);
                singleClick();
            };

            component.clicks++;
            //count clicks
            if (component.clicks === 1) {
                component.previousPoint = component.nodeKey;
                component.timer = setTimeout(setClickevent, component.DELAY);
                return;
            } else if (component.clicks > 1 && component.previousPoint == component.nodeKey) {
                clearTimeout(component.timer);
                //prevent single-click action
                setDblClickEvent();
                component.clicks = 0;
                //after action performed, reset counter
                return;
            }

        }catch(e){
            
        }
        }
    };

    that.trackActionFunc = function(component, type) {

        if (component.customAction !== null) {
            return;
        }

        var isValid = true;
        // precautionary condtion
        // if the action (drilldown, expand, select) already exist for a node in trackAction then don't track;
        if (component.trackAction.length && ['drilldown', 'expand', 'select'].indexOf(type) !== -1) {
            var lastAction = component.trackAction[component.trackAction.length - 1];
            isValid = false;
            for (var key in lastAction) {
                if (lastAction[key] != component[key]) {
                    isValid = true;
                    break;
                }
            }
        }

        if (isValid) {
            var typeFunc = {
                "drilldown": function() {
                    component.trackAction.push({
                        "nodeKey": component.nodeKey,
                        "seriesColumnkey": component.seriesColumnkey,
                        "parentKey": component.parentKey,
                        "nodeLevel": component.nodeLevel,
                        "action": 'drilldown'
                    });
                },
                "drillup": function() {
                    // Remove all the actions in trackAction untill drilldown is encountered.
                    while (component.trackAction.length) {
                        var act = component.trackAction.pop();
                        if (act["action"] == "drilldown") {
                            break;
                        }
                    }
                },
                "expand": function() {
                    component.trackAction.push({
                        "nodeKey": component.nodeKey,
                        "seriesColumnkey": component.seriesColumnkey,
                        "nodeLevel": component.nodeLevel,
                        "parentKey": component.parentKey,
                        "action": 'expand'
                    });
                },
                "collapse": function() {
                    // Remove all the actions of children of collapsed member in trackAction untill collapsed member is encountered.
                    var i = component.trackAction.length - 1;
                    while (i >= 0) {
                        var act = component.trackAction[i];
                        if (act["parentKey"] == component.nodeKey || (act["action"] == "expand" && act["nodeKey"] == component.nodeKey)) {
                            component.trackAction.splice(i, 1);
                            if (act["nodeKey"] == component.nodeKey) {
                                break;
                            }
                        }
                        i--;
                    }
                },
                "select": function() {
                    component.trackAction.push({
                        "nodeKey": component.nodeKey,
                        "seriesColumnkey": component.seriesColumnkey,
                        "parentKey": component.parentKey,
                        "nodeLevel": component.nodeLevel,
                        "action": 'select'
                    });

                },
                "deselect": function() {
                    // Remove action that correspond to deselected node
                    var act = component.trackAction.pop();
                    if (act["nodeKey"] != component.nodeKey || act["action"] != "select" || act["seriesColumnkey"] !== component.seriesColumnkey) {
                        component.trackAction.push(act);
                    }
                }
            };

            typeFunc[type]();
            //console.log(component.trackAction);
        }


    };


    // rebuildActionsRecursive is made recursive by calling it from hierarchyDrilldownFromJson() & expandNode()
    that.rebuildActionsRecursive = function(component) {
        if (!component.trackActionBkp) {
            component.trackActionBkp = [];
            component.trackActionBkp = component.trackActionBkp.concat(component.trackAction);
        }

        var trackActionBkp = component.trackActionBkp;
        if (trackActionBkp.length) {
            var act = trackActionBkp.shift(),
				chart = component.chartRef,
                series = chart.series,
                seriesRef = "",
                pointRef = "";

            for (var j = 0, len = series.length; j < len; j++) {
                if (series[j].data[0] && series[j].data[0].columnKey == act['seriesColumnkey']) {
                    seriesRef = series[j];
                    break;
                }
            }
            if (seriesRef) {
                for (var j = 0, len = seriesRef.data.length; j < len; j++) {
                    if (seriesRef.data[j].Namekey == act['nodeKey']) {
                        pointRef = seriesRef.data[j];
                        break;
                    }
                }

                if (pointRef) {
                    component.customAction = act['action'];
                    pointRef.firePointEvent('click');
                }
            }

        } else {
            component.customAction = null;
            component.trackActionBkp = null;
            //component.chartRef.redraw();
        }
    };

    that.pointTriggeredClick = function(event, component) {
        component.eventPoint = event.target;
        component.nodeLevel = component.eventPoint.level;
        component.selectedlevel(Number(component.nodeLevel));
        component.seriesColumnkey = component.eventPoint.columnKey;
        component.nodeKey = component.eventPoint.Namekey;
        component.parentKey = component.eventPoint.parentKey;
        if (component.customAction == "expand") {
            if (component.eventPoint.Namekey) {
                component.eventPoint.select(!1, !1);
                that.expandCollapseHandler(component);
            }
        } else if (component.customAction == "drilldown") {
            if (!component.eventPoint.seriesOptions && component.isHierarchy) {
                if (typeof component.drilldownjson[component.nodeKey + '-' + component.seriesColumnkey] == 'undefined') {
                    component.selectednode(component.nodeKey);
                    component.selecteddimension(component.dataRes.dimensions[component.selectedDimension].key);
                    component.firePropertiesChangedAndEvent([
                        'selectednode',
                        'selecteddimension'
                    ], 'onHExpand');
                    component.drilldownflag = 1;
                } else {
                    that.hierarchyDrilldownFromJson(component);
                }
            } else {
                that.hierarchyDrilldownFromJson(component);
            }
        } else if (component.customAction == "select") {
            component.eventPoint.select(!0, !0);
            that.rebuildActionsRecursive(component);
        }
    };
	
	that.getCategoryLength = function(category, count){
		var self = this;
		if(typeof count === "undefined"){
			count = 0;
		}
		if(category.length){
			category.forEach(function(cat){
				if(typeof cat == "object"){
					count+=self.getCategoryLength(cat.categories || []);
				}
				else
				{
					count+=1;
				}
			});
		}		
		return count;		
	};
	
	that.updateCategoryWidthHeight = function(component, category){
		if(!component.scrollbarenabled()){
			return;
		}		
		var self = this;
		var chart = component.chartRef;		
		var catLen = 1;		
		var wh = [null, null];
		var type = component.columncharttype().toLowerCase();
		var index = (type == 'bar' ? 1 : 0);
		var chartWidth = component.chartwidth();
		if(chartWidth){
			if(index){
				wh[index] = chartWidth;
			}
			else
			{
				wh[0] = chartWidth;
			}
			chart.setSize(wh[0], wh[1], false);
		}
		else
		{
			catLen = self.getCategoryLength(category);
			if(component.singleCategoryWidthHeight == 1){
				component.chartOriginalWidthHeight = index ? chart.chartHeight : chart.chartWidth;
				component.singleCategoryWidthHeight = component.chartOriginalWidthHeight / catLen;
			}
			else{
				wh[index] = component.singleCategoryWidthHeight * catLen;	
				wh[index] = wh[index] < component.chartOriginalWidthHeight ? component.chartOriginalWidthHeight : wh[index];
				chart.setSize(wh[0], wh[1], false);
			}
		}										
    };

    /**
     * @param {Object} obj - The object which needs to be augmented with properties
     * @param {string} path - The string which represents the '.' separated properties to access the value Eg. 'title.text' or 'xAxis.labels.enabled'
     * @param {*} value - Value to set at the mentioned path
     */
    that.addProperty = function addProperty(obj, path, value){
        var self = this;
        if(!obj){
            return;
        }
        var keys = path.split('.');
        var lastKey = keys.pop();
        var parent = obj;
        _.each(keys, function(key){
            parent =  self.assignValue(parent, key);   
        });
        self.assignValue(parent, lastKey, value);
    };


    that.assignValue = function assignValue(parent, key, value){
        var regex = /([\w\d]*)\[(\d*)\]/g;
        var val = regex.exec(key);
        var index;
        if(val){ // is Array Reference
        key = val[1];
        index = Number(val[2]);
        if(typeof parent[key] === 'undefined' || !_.isArray(parent[key])){
                parent[key] = [];
            }        
            if(typeof parent[key][index] === 'undefined'){
                parent[key][index] = typeof value !== 'undefined' ? value : {}; //Object.create(null,{});
            } 
            parent = parent[key][index];        
        }
        else{
            if(typeof parent[key] === 'undefined'){
                parent[key] = typeof value !== 'undefined' ? value : {}; //Object.create(null,{});
            }
            parent = parent[key];
        }
        return parent;
    };
    /**
     * This function serves dual purpose of providing 
     * 1) All the highcharts related scriptable properties
     * 2) Custum function for each property to update Highcharts Object
     * @return {Object | Array} If function is provided with parameters then hcprops object will be returned otherwise only keys in hcprops will be returned
     */
    that.getScriptableProperties = function(params){
        var self = this;
        var component;
        var hcOptions;
        var vbiHighchartsDefault;
        var vbiNumberFormat;
        var decimalsep;
        var thousandsep;
        var thousandseptool;
        var decimalseptool;
        if(params){
            component = params.component;
            hcOptions = params.hcOptions;
            vbiHighchartsDefault = params.vbiHighchartsDefault;
            vbiNumberFormat = params.vbiNumberFormat;
            decimalsep = component.bulletdecimalseparator();
	        thousandsep = component.bulletthousandseparator();
            thousandseptool = component.tooltipthousandseparator();
            decimalseptool = component.tooltipdecimalseparator();
        }
        var hcprops = {
            'columncharttype': function () {
                var chartType = component.columncharttype().toLowerCase();
                var newVal = chartType == 'bar' ? true : false;			
                self.addProperty(hcOptions, 'chart.inverted', newVal);
            },

            'chartwidth': function () {
                var newVal = Number(component.chartwidth());
                var chartType = component.columncharttype().toLowerCase();
                if (component.scrollbarenabled() && newVal) {
                    if (chartType === "bar") {					
                        self.addProperty(hcOptions, 'chart.height', newVal);
                    } else {					
                        self.addProperty(hcOptions, 'chart.width', newVal);
                    }
                }
            },

            'datalabelenabled': function () {
                var newVal = component.datalabelenabled();			
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.enabled', newVal);
            },

            // X-Axis
            'xaxisplotbandcolor': function () {
                var newVal = component.xaxisplotbandenabled() === true ? component.xaxisplotbandcolor() : null;
                self.addProperty(hcOptions, 'xAxis[0].plotBands[0].color', newVal);
            },

            'xaxisplotbandenabled': function () {
                this.xaxisplotbandcolor();
            },

            'xaxisplotbandfrom': function () {
                var newVal = component.xaxisplotbandfrom();
                self.addProperty(hcOptions, 'xAxis[0].plotBands[0].from', newVal - 0.5);
            },

            'xaxisplotbandto': function () {
                var newVal = component.xaxisplotbandto();
                self.addProperty(hcOptions, 'xAxis[0].plotBands[0].to', newVal + 0.5);
            },

            'xaxisplotlinecolor': function () {
                var newVal = component.xaxisplotlineenabled() === true ? component.xaxisplotlinecolor() : null;
                self.addProperty(hcOptions, 'options.xAxis[0].plotLines[0].color', newVal);
            },

            'xaxisplotlineenabled': function () {
                this.xaxisplotlinecolor();
            },

            'xaxisplotlinevalue': function () {
                var newVal = component.xaxisplotlinevalue();
                self.addProperty(hcOptions, 'options.xAxis[0].plotLines[0].value', newVal);
            },

            'xaxistitleenabled': function () {
                var newVal = component.xaxistitleenabled() ? 'block' : 'none';
                self.addProperty(hcOptions, 'xAxis[0].title.style.display', newVal);
            },

            'xaxistitletext': function () {
                var newVal = component.xaxistitletext();
                self.addProperty(hcOptions, 'xAxis[0].title.text', newVal);
            },

            'xaxislabelenabled': function () {
                var newVal = component.xaxislabelenabled();
                self.addProperty(hcOptions, 'xAxis[0].labels.enabled', newVal);
            },

            'xaxislabelformat': function () {
                var newVal = component.xaxislabelprefix() + component.xaxislabelformat() + component.xaxislabelsuffix();
                self.addProperty(hcOptions, 'xAxis[0].labels.format', newVal);
            },

            // Y-Axis
            'yaxisplotbandcolor': function () {
                var newVal = component.yaxisplotbandcolor();
                self.addProperty(hcOptions, 'yAxis[0].plotBands[0].color', newVal);
            },

            'yaxisplotbandenabled': function () {
                this.yaxisplotbandcolor();
            },

            'yaxisplotbandfrom': function () {
                var newVal = component.yaxisplotbandfrom();
                self.addProperty(hcOptions, 'yAxis[0].plotBands[0].from', newVal);
            },

            'yaxisplotbandto': function () {
                var newVal = component.yaxisplotbandto();
                self.addProperty(hcOptions, 'yAxis[0].plotBands[0].to', newVal);
            },

            'yaxisplotlinecolor': function () {
                var newVal = component.yaxisplotlineenabled() === true ? component.yaxisplotlinecolor() : null;
                self.addProperty(hcOptions, 'yAxis[0].plotLines[0].color', newVal);
            },

            'yaxisplotlineenabled': function () {
                this.yaxisplotlinecolor();
            },

            'yaxisplotlinevalue': function () {
                var newVal = component.yaxisplotlinevalue();
                self.addProperty(hcOptions, 'yAxis[0].plotLines[0].value', newVal);
            },

            'yaxistitleenabled': function () {
                var newVal = component.yaxistitleenabled() ? 'block' : 'none';
                self.addProperty(hcOptions, 'yAxis[0].title.style.display', newVal);
            },

            'yaxistitletext': function () {
                var newVal = component.yaxistitletext();
                self.addProperty(hcOptions, 'yAxis[0].title.text', newVal);
            },

            'yaxislabelenabled': function () {
                var newVal = component.yaxislabelenabled();
                self.addProperty(hcOptions, 'yAxis[0].labels.enabled', newVal);
            },

            'yaxismin': function () {
                var newVal = component.yaxismin() === 0 ? null : _.parseInt(component.yaxismin());
                self.addProperty(hcOptions, 'yAxis[0].min', newVal);
            },
            'yaxismax': function () {
                var newVal = component.yaxismax() === 0 ? null : _.parseInt(component.yaxismax());
                self.addProperty(hcOptions, 'yAxis[0].max', newVal);
            },

            // Title
            'titletext': function () {
                var newVal = component.titletext() === undefined ? '' : component.titletext();
                self.addProperty(hcOptions, 'title.text', newVal);
            },

            'titleenable': function () {
                var newVal = component.titleenable() === true ? 'block' : 'none';
                self.addProperty(hcOptions, 'title.style.display', newVal);
            },

            // Subtitle
            'subtitletext': function () {
                var newVal = component.subtitletext() === undefined ? '' : component.subtitletext();
                self.addProperty(hcOptions, 'subtitle.text', newVal);
            },

            'subtitleenable': function () {
                var newVal = component.subtitleenable() === true ? 'block' : 'none';			
                self.addProperty(hcOptions, 'subtitle.style.display', newVal);			
            },

            //Tooltip
            'tooltipenable': function () {
                self.addProperty(hcOptions, 'tooltip.formatter', tooltipFormatter);
            },

            'tooltipvaluedecimals': function () {
                self.addProperty(hcOptions, 'tooltip.formatter', tooltipFormatter);
            },

            'tooltipvalueprefix': function () {
                self.addProperty(hcOptions, 'tooltip.formatter', tooltipFormatter);
            },

            'tooltipvaluesuffix': function () {
                self.addProperty(hcOptions, 'tooltip.formatter', tooltipFormatter);
            },

            // Export
            'exportedfilename': function () {},

            // DataLabel
            'datalabelprefix': function () {
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.formatter', dataLabelFormatter);
            },
            'datalabelsuffix': function () {
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.formatter', dataLabelFormatter);
            },
            'bulletdecimalseparator': function () {
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.formatter', dataLabelFormatter);
            },
            'bulletthousandseparator': function () {
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.formatter', dataLabelFormatter);
            },
            'decimal': function () {
                self.addProperty(hcOptions, 'plotOptions.series.dataLabels.formatter', dataLabelFormatter);
            },

            //Y-Axis
            'yaxislabelprefix': function () {
                self.addProperty(hcOptions, 'yAxis.labels.formatter', yAxisLabelFormatter);
            },

            'yaxislabelsuffix': function () {
                self.addProperty(hcOptions, 'yAxis.labels.formatter', yAxisLabelFormatter);
            }

        };

        return params ? hcprops : Object.keys(hcprops);
        
        // formatter functions        
        function tooltipFormatter() {
            if(component.customtooltip() !== ""  && Object.keys(JSON.parse(component.customtooltip()||'{}')).length){
                return vbiNumberFormat.DrillDownFamilyTooltipFormatter(this,component,decimalseptool,thousandseptool,false);
            }else {
                return vbiNumberFormat.piedrilldownFamilyTooltip(this,component,decimalseptool,thousandseptool);
            }
        }
        function dataLabelFormatter() {
            try {
                if (component.customdatalabel() !== "" && Object.keys(JSON.parse(component.customdatalabel() || '{}')).length) {
                    return vbiNumberFormat.drillDownFamilyFormatterDataLabel(this, component, decimalsep, decimalsep, false);
                } else {
                    return vbiNumberFormat.piedrilldownFamilyDataLabelFormatter(this, component, decimalsep, thousandsep);
                }
            } catch (e) {}
        }
        function yAxisLabelFormatter() {
            return vbiHighchartsDefault['Yaxis_autoscale_formatter'](component,this);
        }
    };

    /**
     * function to update oControlPropertiesHcProps object with the values in oControlProperties.
     * oControlPropertiesHcProps will be used in updateHighchartsProp function to check if a value of property has changed.
     * This function should be executed at the end of afterUpdate becuase updateHighchartsProp will be called somewhere in the middle.
     * 
     */
    that.updateOcontrolPropertiesHcProps = function(component){
        var self = this;
        var properties = self.getScriptableProperties();
        if(!component.oControlPropertiesHcProps){
            component.oControlPropertiesHcProps = {};
        }
        _.each(properties, function(props){
              component.oControlPropertiesHcProps[props] = component.oControlProperties[props];
        });
    };


    /**
     * Function to update the chart without refresh/reload. The feature is limited to highchart related properties
     */
    that.updateHighchartsProp = function (component, vbiNumberFormat, vbiHighchartsDefault) {
        var self = this;
        var chart = component.chartRef;
        var options = chart.options;
        var hasChanged = 0;
        var hcOptions = {}; //Object.create(null,{});
        var highchartProperties = [];
        var hcprops = self.getScriptableProperties({
            component: component,
            hcOptions: hcOptions,
            vbiHighchartsDefault: vbiHighchartsDefault,
            vbiNumberFormat: vbiNumberFormat
        });        
        try{
            _.each(Object.keys(hcprops), function(props){
                if(component.oControlPropertiesHcProps && component.oControlProperties[props] !== component.oControlPropertiesHcProps[props]){
                    hcprops[props]();
                }
            });
        }catch(e){
            //console.log(e);
        }

        if (Object.keys(hcOptions).length) {
            setTimeout(function(){
                chart.update(hcOptions);
            },500); //Timeout is added so that the update is smooth after drilldown or expand/collapse
            
            //component.afterUpdate();
        }
    };

}
