/* global  vbiGlobalMaps */
//VbiMaps Library

//VisualBI Solutions
//http://www.visualbis.com

function VbiMaps(component) {

  if (component !== undefined) {
    this.component = component;
  }

  this.baseUrl = 'zen/mimes/sdk_include/com.visualbi.maps/res/maps';  // 'http://cdn.visualbis.com/maps';
  this.extension = '.js';

  this.maps = map_sources;
}

VbiMaps.prototype.getMapSource = function(map, remote, respCallback) {
  //console.log(remote);
  //check on params
  if (map === undefined) { return; }

  var self = this;
  var obj = map.obj;

  //If offline then return from
  if (map.offline) {
    if (respCallback !== undefined) {
      respCallback(Highcharts.maps[obj]);
      return;
    }
  } else {
    //Check if the map is already loaded
    if (Highcharts.maps[obj] !== undefined) {
      respCallback(Highcharts.maps[obj]);
      return;
    }

    if (remote === undefined || remote === '') {
      remote = self.baseUrl;
    }

    //Check for trailing slash
    var url = remote + '/' + obj + self.extension;
    if (remote[remote.length - 1] === '/') {
      url = remote + obj + self.extension;
    }

    var callback = function(data) {
      respCallback(data);
      return;
    };

    try {
      $.getScript(url)
			  .done(function(script, textStatus) {
  if (Highcharts.maps[obj] === undefined && remote !== self.baseUrl) { 	//Catch errors (handle) with the CDN
    alert('Sorry an error occurred loading the map at ' + url + ', please verify the map url. The reason given as: File not found (404)');
    respCallback(false);
    return;
  } else if (Highcharts.maps[obj] === undefined) {
    if (window.console) {
      console.log('Sorry an error occurred loading the map at ' + url + ', please verify the map url. The reason given as: File not found (404)');
    }

    respCallback(false);
    return;
  }

  var map_data = Highcharts.maps[obj];		//Once the map is retrieved send the highcharts obj back
  if (respCallback !== undefined) {
    respCallback(map_data);
    return;
  }
			  })
			  .fail(function(jqxhr, settings, exception) {
  if (remote !== self.baseUrl) {	 //Catch errors with the CDN
    alert('Sorry an error occurred loading the map at ' + url + ', please verify the map url. The reason given as - ' + JSON.stringify(exception));
  }
  //console.log("Loading map from source failed. The reason given as: " + JSON.stringify(msg));
  respCallback(false);
  return;
			});
    }
    catch (err) {
      alert('Sorry an error occurred loading the map at ' + url + ', please verify the map url. The reason given as - ' + err);
      respCallback(false);
      return;
    }

  }
};

VbiMaps.prototype.detectFormat = function(geoName) {

  if (/^[a-zA-Z]{2}$/.test(geoName)) {
    return 'hc-a2';
  } else if (/^[a-zA-Z]{3}$/.test(geoName)) {
    return 'iso-a3';
  } else if (/^[a-zA-Z]{2}[0-9]{2}$/.test(geoName)) {
    return 'fips';
  } else if (/^[0-9]{5}$/.test(geoName)) {
    return 'fips';
  }
  //else if(/^[a-zA-Z]{2}[-#,.\/][a-zA-Z]{2}$/.test(geoName)){
  else if (/^[a-zA-Z]{2}[#,.\/][a-zA-Z]{2}$/.test(geoName)) {
    return 'hasc';
  } else if (/([A-z]{2})((-)(([A-z]{2})|([0-9]+)))+/.test(geoName)) { //Pattern for hc-key (us-tx, us-285, us-tx-2482)
    return 'hc-key';
  } else if (/([0-9]{4})/.test(geoName) && geoName.toString().length === 4) { //Pattern for geod id (4 digits)
    return 'geoid';
  } else if (/(-)*[0-9]+$/.test(geoName)) {
    return 'woe-id';
  } else if (/^[a-zsA-Z]*$/g.test(geoName)) {
    return 'name';
  } else {
    return 'name'; //Unknown data format
  }
};

VbiMaps.prototype.convertCase = function(geoName, format) {
  //	console.log(geoName);
  if (format === 'hc-a2') {
    return geoName.toUpperCase();
  } else if (format === 'iso-a3') {
    return geoName.toUpperCase();
  } else if (format === 'fips') {
    return geoName.toUpperCase();
  } else if (format === 'hc-key') { 	//lowercase for hc-key
    if(/^(r|R)(u|U)( )(r|R)(u|U)(-)[A-z]*$/.test(geoName)){
        return  geoName.split(' ')[1].substr(0, 5).toLowerCase();
    }//Russia support ru-al..
    else{
      return geoName.toLowerCase();
    }
  } else if (format === 'hasc') {
    //replace the middle special character
    geoName = geoName.replace(/[-#,\/]/g,'.');
    geoName = geoName.toUpperCase();
    return geoName;
  } else if (format === 'name') {
    //split into words
    var words = geoName.split(' ');

    for (var i = 0; i < words.length; i++) {

      words[i] = words[i].toLowerCase();

      if (words[i] === 'the' || words[i] === 'of' || words[i] === 'de' || words[i] === 'a' || words[i] === '') {
        continue;
      }

      words[i] = words[i][0].toUpperCase() + words[i].substring(1);
    }

    return words.join(' ');
  }

  return geoName;
};

VbiMaps.prototype.modifyStateName = function(dataSource) {
  //US State Object
  var usStates = {'AL': 'Alabama', 'AK': 'Alaska', 'AS': 'American Samoa', 'AZ': 'Arizona', 'AR': 'Arkansas', 'CA': 'California', 'CO': 'Colorado', 'CT': 'Connecticut', 'DE': 'Delaware', 'DC': 'District of Columbia', 'FL': 'Florida', 'GA': 'Georgia', 'GU': 'Guam', 'HI': 'Hawaii', 'ID': 'Idaho', 'IL': 'Illinois', 'IN': 'Indiana', 'IA': 'Iowa', 'KS': 'Kansas', 'KY': 'Kentucky', 'LA': 'Louisiana', 'ME': 'Maine', 'MD': 'Maryland', 'MA': 'Massachusetts', 'MI': 'Michigan', 'MN': 'Minnesota', 'MS': 'Mississippi', 'MO': 'Missouri', 'MT': 'Montana', 'NE': 'Nebraska', 'NV': 'Nevada', 'NH': 'New Hampshire', 'NJ': 'New Jersey', 'NM': 'New Mexico', 'NY': 'New York', 'NC': 'North Carolina', 'ND': 'North Dakota', 'MP': 'Northern Mariana Islands', 'OH': 'Ohio', 'OK': 'Oklahoma', 'OR': 'Oregon', 'PA': 'Pennsylvania', 'PR': 'Puerto Rico', 'RI': 'Rhode Island', 'SC': 'South Carolina', 'SD': 'South Dakota', 'TN': 'Tennessee', 'TX': 'Texas', 'UM': 'United States Minor Outlying Islands', 'UT': 'Utah', 'VT': 'Vermont', 'VA': 'Virginia', 'VI': 'Virgin Islands', 'WA': 'Washington', 'WV': 'West Virginia', 'WI': 'Wisconsin', 'WY': 'Wyoming'};
  var usFips = {'01': 'AL', '02': 'AK', '60': 'AS', '04': 'AZ', '05': 'AR', '06': 'CA', '08': 'CO', '09': 'CT', '10': 'DE', '11': 'DC', '12': 'FL', '13': 'GA', '66': 'GU', '15': 'HI', '16': 'ID', '17': 'IL', '18': 'IN', '19': 'IA', '20': 'KS', '21': 'KY', '22': 'LA', '23': 'ME', '24': 'MD', '25': 'MA', '26': 'MI', '27': 'MN', '28': 'MS', '29': 'MO', '30': 'MT', '31': 'NE', '32': 'NV', '33': 'NH', '34': 'NJ', '35': 'NM', '36': 'NY', '37': 'NC', '38': 'ND', '39': 'OH', '40': 'OK', '41': 'OR', '42': 'PA', '72': 'PR', '44': 'RI', '45': 'SC', '46': 'SD', '47': 'TN', '48': 'TX', '49': 'UT', '50': 'VT', '51': 'VA', '78': 'VI', '53': 'WA', '54': 'WV', '55': 'WI', '56': 'WY'};

  var statesName = [];

  //Pusing the states from Object into array
  for (var key in usStates) {
    statesName.push(key.toLowerCase());
    statesName.push(usStates[key].toLowerCase());
  }

  //Iterate over the datasource
  for (var i = 0; i < dataSource.length; i++) {
    //If the data contains (- or /) and (us) word mentioned,  catch it and store code to the array.
    if (dataSource[i].search(/-|\//) != -1 && dataSource[i].toLowerCase().search(/us/) != -1) {
      dataSource[i] = dataSource[i].toLowerCase().split(/-|\//)[1].toUpperCase();
    }

    //Find through Federal Information Processing Standards (FIPS)
    if (dataSource[i].search(/\d+/) != -1) {
      var fips = parseInt(dataSource[i].match(/\d+/));
      //All FIPS code is of length 2 & its a number.  Just a validation.
      if (typeof fips == 'number' && fips.toString().length == 2) {
        dataSource[i] = usFips[fips];
      }
    }

    //Check whether the data in exists in states array or not
    if (vbiGlobalMaps.arrayContains(dataSource[i].toLowerCase(), statesName)) {
      //If the mentioned item is not code then get code from the us state object.
      if (dataSource[i].toLowerCase().length > 2) {
        for (var key in usStates) {
          if (usStates[key].toLowerCase() == dataSource[i].toLowerCase()) {
            dataSource[i] = key.toUpperCase();
            break;
          }
        }
      }
    }
  }

  return dataSource;
};

//Javascript function to check whether the value present in the array or not
VbiMaps.prototype.arrayContains = function(value, array) {
  return (array.indexOf(value) > -1);
};

//If already defined then dont sweat (execute JS further)
if (vbiGlobalMaps === undefined) {
  var vbiGlobalMaps = new VbiMaps();
}
